unit oCPAccessControlList;

interface

uses
  Dialogs,
  Classes,
  ComCtrls,
  uCPXML,
  oInterfaces,
  oCPTypes,
  oCP_Exceptions,
  SysUtils,
  XMLIntf,
  XMLDoc;

type
  TCPAccessControlList = class(TInterfacedObject, ICPAccessControlList)
  private
    fDBConnection: IDBConnection;
    fPersonService: IPersonService;
    fItems: IInterfaceList;
    fLoaded: boolean;
  public
    constructor Create(aDBConnection: IDBConnection; aPersonService: IPersonService);
    destructor Destroy; override;

    function Item(aIndex: Integer): ICPAccessControlListItem; stdcall;
    function ItemCount: integer; stdcall;
    function ItemAccessLevel(aItemID, aUserID: string): TCPAccessLevel; stdcall;

    procedure Load; stdcall;
    procedure LoadList(aItemID: string; aList: TStrings); stdcall;
    procedure LoadListView(aItemID: string; aListView: TListView); stdcall;
    procedure Update(aItemID, aUserID: string; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure Update(aACLItem: ICPAccessControlListItem; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure Delete(aItemID, aUserID: string); stdcall;
    procedure DeleteAll(aItemID: string); stdcall;
  end;

implementation

uses
  oCPAccessControlListItem;

{ TCPAccessControlList }

constructor TCPAccessControlList.Create(aDBConnection: IDBConnection; aPersonService: IPersonService);
begin
  fDBConnection := aDBConnection;
  fPersonService := aPersonService;
  fLoaded := False;
  TInterfaceList.Create.GetInterface(IInterfaceList, fItems);
end;

destructor TCPAccessControlList.Destroy;
begin
  fDBConnection := nil;
  fPersonService := nil;
  fItems.Clear;
  fItems := nil;
  inherited;
end;

function TCPAccessControlList.Item(aIndex: Integer): ICPAccessControlListItem;
begin
  try
    fItems[aIndex].QueryInterface(ICPAccessControlListItem, Result);
  except
    Result := nil;
  end;
end;

function TCPAccessControlList.ItemAccessLevel(aItemID, aUserID: string): TCPAccessLevel;
var
  i: integer;
  aItem: ICPAccessControlListItem;
begin
  Result := alReadOnly; {Default}
  for i := 0 to fItems.Count - 1 do
    if fItems[i].QueryInterface(ICPAccessControlListItem, aItem) = 0 then
      if aItem.getItemID = aItemID then
        if aItem.getUserID = aUserID then
          begin
            Result := aItem.getAccessLevel;
            Exit;
          end;
end;

function TCPAccessControlList.ItemCount: integer;
begin
  Result := fItems.Count;
end;

procedure TCPAccessControlList.Update(aItemID, aUserID: string; aAccessLevel: TCPAccessLevel);
var
  i: integer;
  aItem: ICPAccessControlListItem;
begin
  { Either find it in the list or create a new one then call the 'other' Update() }

  for i := 0 to fItems.Count - 1 do
    begin
      fItems[i].QueryInterface(ICPAccessControlListItem, aItem);
      if (aItem.getItemID = aItemID) and (aItem.getUserID = aUserID) then
        begin
          Update(aItem, aAccessLevel);
          Exit;
        end;
    end;

  if TCPAccessControlListItem.Create(aItemID, aUserID, aAccessLevel).GetInterface(ICPAccessControlListItem, aItem) then
    begin
      fItems.Add(aItem);
      Update(aItem, aAccessLevel);
    end;
end;

procedure TCPAccessControlList.Update(aACLItem: ICPAccessControlListItem; aAccessLevel: TCPAccessLevel);
begin
  if fDBConnection.ExecCommand('UpdateACLPermission', [aACLItem.getItemID, aACLItem.getUserID, aAccessLevel]) then
    aACLItem.setAccessLevel(aAccessLevel);
end;

procedure TCPAccessControlList.Delete(aItemID, aUserID: string);
var
  i: integer;
  aItem: ICPAccessControlListItem;
begin
  if fDBConnection.ExecCommand('RemoveACLPermission', [aItemID, aUserID]) then
    for i := fItems.Count - 1 downto 0 do
      if fItems[i].QueryInterface(ICPAccessControlListItem, aItem) = 0 then
        if (aItem.getItemID = aItemID) and (aItem.getUserID = aUserID) then
          fItems.Delete(i);
end;

procedure TCPAccessControlList.DeleteAll(aItemID: string);
var
  i: integer;
  aItem: ICPAccessControlListItem;
begin
  if fDBConnection.ExecCommand('RemoveACLPermission', [aItemID]) then
    for i := fItems.Count - 1 downto 0 do
      if fItems[i].QueryInterface(ICPAccessControlListItem, aItem) = 0 then
        if (aItem.getItemID = aItemID) then
          fItems.Delete(i);
end;

procedure TCPAccessControlList.Load;
var
  aItem: IInterface;
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
begin
  fItems.Clear;
  aXMLDoc := NewXMLDocument;

  if not fDBConnection.ExecQuery('GetAccessControlList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting Access Control List.');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      TCPAccessControlListItem.Create(aXMLNode).GetInterface(ICPAccessControlListItem, aItem);
      if Assigned(aItem) then
        fItems.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;
end;

procedure TCPAccessControlList.LoadList(aItemID: string; aList: TStrings);
var
  i: integer;
  aItem: ICPAccessControlListItem;
  aPerson: IPerson;
  aCaption: string;
begin
  aList.Clear;
  for i := 0 to fItems.Count - 1 do
    if fItems[i].QueryInterface(ICPAccessControlListItem, aItem) = 0 then
      if aItem.getItemID = aItemID then
        begin
          aPerson := fPersonService.getPerson(aItem.getUserID);
          aCaption := Format('%s (%s)', [aPerson.getName, aPerson.getTitle]);
          aItem.AddToList(aList, aCaption);
        end;
end;

procedure TCPAccessControlList.LoadListView(aItemID: string; aListView: TListView);
var
  i: integer;
  aItem: ICPAccessControlListItem;
  aPerson: IPerson;
begin
  aListView.Items.BeginUpdate;
  try
    aListView.Clear;
    for i := 0 to fItems.Count - 1 do
      if fItems[i].QueryInterface(ICPAccessControlListItem, aItem) = 0 then
        if aItem.getItemID = aItemID then
          with aItem.AddToListView(aListView) do
            begin
              aPerson := fPersonService.getPerson(aItem.getUserID);
              Caption := Format('%s (%s)', [aPerson.getName, aPerson.getTitle]);
              SubItems.Add(CP_ACCESS_LEVEL[aItem.getAccessLevel]);
            end;
  finally
    aListView.Items.EndUpdate;
  end;
end;

end.

