unit oCPFlowsheet;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 9 $  $Modtime: 5/22/09 11:17a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary Flowsheet Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPFlowsheet.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  SysUtils,
  XMLDoc,
  XMLIntf;

type
  TCPFlowsheet = class(TInterfacedObject, ICPConsoleItem, ICPFlowsheet)
  private
    [weak] fCPService: ICPService;
    fTerminology: ITerminology;
    fIsLoaded: boolean;
    fActive: boolean;
    fComment: string;
    fID: string;
    fName: string;
    fDisplayName: string;
    fPages: IInterfaceList;
    fTotals: IInterfaceList;
    fTIUNoteID: string;
    fTIUNoteName: string;
    fDisplayKardex: boolean;

    {ICPConsole Item}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;
    function getActive: Boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;
    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;
    procedure Load; stdcall;

    {ICPFlowsheet Interface}
    function getDefaultTIUNoteID: string; stdcall;
    function getDefaultTIUNote: string; stdcall;

    procedure setDefaultTIUNoteID(const aValue: string); stdcall;
    procedure setDefaultTIUNote(const aValue: string); stdcall;

    function getDisplayKardex: boolean; stdcall;
    procedure setDisplayKardex(const aValue: boolean); stdcall;

    function Page(aIndex: integer): ICPFlowsheetPage; stdcall;
    function PageCount: integer; stdcall;

    function Total(aIndex: integer): ICPFlowsheetTotal; stdcall;
    function TotalCount: integer; stdcall;

    function LoadPages(aList: TStrings): integer; overload; stdcall;
    function LoadPages(aListView: TListView): integer; overload; stdcall;

    procedure ClearPages; stdcall;
    procedure InsertPage(aPage: ICPFlowsheetPage); stdcall;
    procedure RemovePage(aPage: ICPFlowsheetPage); stdcall;

    function LoadTotals(aList: TStrings): integer; overload; stdcall;
    function LoadTotals(aListView: TListView): integer; overload; stdcall;

    procedure ClearTotals; stdcall;
    procedure InsertTotal(aTotal: ICPFlowsheetTotal); stdcall;
    procedure RemoveTotal(aTotal: ICPFlowsheetTotal); stdcall;

    {Internal Stuff}
    function LoadFromXML(aXMLNode: IXMLNode): Boolean;
    procedure ReLoadPages;
    procedure ReLoadTotals;
    procedure Save; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPFlowsheet(aXMLNode: IXMLNode; aCPService: ICPService; aTerminology: ITerminology): TCPFlowsheet;

implementation

uses
  oCP_Exceptions,
  oCPFlowsheetPage,
  oCPFlowsheetTotal,
  uCPXML;

function NewCPFlowsheet(aXMLNode: IXMLNode; aCPService: ICPService; aTerminology: ITerminology): TCPFlowsheet;
begin
  Result := TCPFlowsheet.Create;
  with Result do
    try
      fCPService := aCPService;
      fTerminology := aTerminology;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand('AddFlowsheet', [fID, fName, fDisplayName, fComment, fActive]);
    except
      FreeAndNil(Result);
    end;
end;

constructor TCPFlowsheet.Create;
begin
  inherited Create;
  fID := NewGUID;
  fName := 'NEW FLOWSHEET';
  fDisplayName := 'New Flowsheet';
  fComment := 'Comment';
  fActive := False;
  fTIUNoteID := '';
  fTIUNoteName := '';

  TInterfaceList.Create.GetInterface(IInterfaceList, fPages);
  TInterfaceList.Create.GetInterface(IInterfaceList, fTotals);
end;

destructor TCPFlowsheet.Destroy;
begin
  fPages.Clear;
  fTotals.Clear;
  fCPService := nil;
  fTerminology := nil;
  inherited;
end;

procedure TCPFlowsheet.Save;
var
  i: integer;
//  j: integer;
begin
  with fCPService do
    try
      DBTransactionBegin;

      ExecuteDBCommand('UpdateFlowsheet', [fID, fName, fDisplayName, fComment, fActive, fTIUNoteID, fDisplayKardex]);
      ExecuteDBCommand('DeleteFlowsheetPagePairs', [fID]);
      ExecuteDBCommand('DeleteFlowsheetTotalPairs', [fID]);

      for i := 0 to fPages.Count - 1 do
        with Page(i) do
          ExecuteDBCommand('AddFlowsheetPage',
            [getFlowsheetID, getPageOrder, getFlowsheetViewID, getPageType, getDisplayName, getSpecialInstructions]);

      for i := 0 to fTotals.Count - 1 do
        with Total(i) do
          ExecuteDBCommand('AddFlowsheetTotal', [fID, getID, i + 1]);

      DBTransactionCommit;
    except
      DBTransactionRollback;
      raise;
    end;
end;

procedure TCPFlowsheet.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  try
    fIsLoaded := False;

    fPages.Clear;
    fTotals.Clear;

    aXMLDoc := NewXMLDocument;
    fCPService.ExecuteDBQuery('GetFlowsheet', [fID], aXMLDoc);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    LoadFromXML(aXMLRec);

    ReLoadPages;
    ReLoadTotals;

    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Expection ''%s''', [e.Message]);
  end;
end;

procedure TCPFlowsheet.ReLoadPages;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aPage: ICPFlowsheetPage;
begin
  fPages.Clear;
  aXMLDoc := NewXMLDocument;
  fCPService.ExecuteDBQuery('GetFlowsheetPageList', [fID], aXMLDoc);
  aXMLDoc.Active := True;
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aPage := NewCPFlowsheetPage(aXMLRec, Self, fCPService);
      fPages.Add(aPage);
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;
end;

procedure TCPFlowsheet.ReLoadTotals;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aTotal: ICPFlowsheetTotal;
begin
  fTotals.Clear;
  aXMLDoc := NewXMLDocument;
  fCPService.ExecuteDBQuery('GetFlowsheetTotalList', [fID], aXMLDoc);
  aXMLDoc.Active := True;

  //ShowMessage(aXMLDoc.XML.Text);

  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aTotal := fCPService.CPFlowsheetTotal(XMLValueAsStr(aXMLRec, 'TOTAL_ID'));
      aTotal.Load;
      fTotals.Add(aTotal);
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;
end;

{------------------------------------------------------------------------------}
{------------------------------- private methods ------------------------------}
{------------------------------------------------------------------------------}

procedure TCPFlowsheet.ClearPages;
begin
  fPages.Clear;
end;

procedure TCPFlowsheet.ClearTotals;
begin
  fTotals.Clear;
end;

function TCPFlowsheet.getActive: Boolean;
begin
  Result := fActive;
end;

function TCPFlowsheet.getCanDelete: boolean;
begin
  Result := True;
end;

function TCPFlowsheet.getCanRename: boolean;
begin
  Result := True;
end;

function TCPFlowsheet.getComment: string;
begin
  Result := fComment;
end;

function TCPFlowsheet.getName: string;
begin
  Result := fName;
end;

function TCPFlowsheet.getDefaultTIUNote: string;
begin
  Result := fTIUNoteName;
end;

function TCPFlowsheet.getDefaultTIUNoteID: string;
begin
  Result := fTIUNoteID;
end;

function TCPFlowsheet.getDisplayKardex: boolean;
begin
  Result := fDisplayKardex;
end;

function TCPFlowsheet.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TCPFlowsheet.getID: string;
begin
  Result := fID;
end;

function TCPFlowsheet.getIsLoaded: boolean;
begin
  Result := False;
end;

function TCPFlowsheet.getItemType: TCPItemType;
begin
  Result := ciFlowsheet;
end;

procedure TCPFlowsheet.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPFlowsheet.AddToListView(aListView: TListView);
begin
  //
end;

function TCPFlowsheet.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TCPFlowsheet.Page(aIndex: integer): ICPFlowsheetPage;
begin
  try
    fPages[aIndex].QueryInterface(ICPFlowsheetPage, Result);
  except
    Result := nil;
  end;
end;

function TCPFlowsheet.PageCount: integer;
begin
  Result := fPages.Count;
end;

procedure TCPFlowsheet.RemovePage(aPage: ICPFlowsheetPage);
var
  i: integer;
begin
  for i := fPages.Count - 1 downto 0 do
    if (fPages[i] as ICPFlowsheetPage) = aPage then
      fPages.Delete(i);
end;

procedure TCPFlowsheet.RemoveTotal(aTotal: ICPFlowsheetTotal);
var
  i: integer;
begin
  i := fTotals.IndexOf(aTotal);
  if i > -1 then
    fTotals.Delete(i);
end;

function TCPFlowsheet.LoadPages(aList: TStrings): integer;
var
  i: integer;
begin
  aList.Clear;
  for i := 0 to fPages.Count - 1 do
    Page(i).AddToList(aList);
  result := fPages.Count;
end;

function TCPFlowsheet.LoadPages(aListView: TListView): integer;
var
  i: integer;
begin
  aListView.Items.Clear;
  for i := 0 to fPages.Count - 1 do
    Page(i).AddToListView(aListView);
  result := fPages.Count;
end;

function TCPFlowsheet.LoadTotals(aList: TStrings): integer;
var
  i: integer;
begin
  aList.Clear;
  for i := 0 to TotalCount - 1 do
    Total(i).AddToList(aList);
  result := TotalCount;
end;

function TCPFlowsheet.LoadTotals(aListView: TListView): integer;
var
  i: integer;
begin
  aListView.Items.Clear;
  for i := 0 to TotalCount - 1 do
    Total(i).AddToListView(aListView);
  result := TotalCount;
end;

procedure TCPFlowsheet.InsertPage(aPage: ICPFLowsheetPage);
begin
  fPages.Add(aPage);
end;

procedure TCPFlowsheet.InsertTotal(aTotal: ICPFlowsheetTotal);
begin
  fTotals.Add(aTotal);
end;

{------------------------------------------------------------------------------}
{----------------------------- Public methods ---------------------------------}
{------------------------------------------------------------------------------}

function TCPFlowsheet.LoadFromXML(aXMLNode: IXMLNode): Boolean;
begin
  Result := True; {Assume the best here}
  with aXMLNode do
    try
      fID := XMLValueAsStr(aXMLNode, 'FLOWSHEET_ID');
      fName := XMLValueAsStr(aXMLNode, 'NAME');
      fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
      fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
      fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
      fTIUNoteID := XMLValueAsStr(aXMLNode, 'DEFAULT_TIU_NOTE_ID');
      fTIUNoteName := XMLValueAsStr(aXMLNode, 'DEFAULT_TIU_NOTE');
      fDisplayKardex := (XMLValueAsInt(aXMLNode, 'DISPLAY_KARDEX') = 1);
    except
      Result := False;
    end;
end;

procedure TCPFlowsheet.setActive(const aValue: Boolean);
begin
  fActive := aValue;
end;

procedure TCPFlowsheet.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TCPFlowsheet.setDefaultTIUNote(const aValue: string);
begin
  fTIUNoteName := aValue;
end;

procedure TCPFlowsheet.setDefaultTIUNoteID(const aValue: string);
begin
  fTIUNoteID := aValue;
end;

procedure TCPFlowsheet.setDisplayKardex(const aValue: boolean);
begin
  fDisplayKardex := aValue;
end;

procedure TCPFlowsheet.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TCPFlowsheet.setName(const aValue: string);
begin
  fName := aValue;
end;

function TCPFlowsheet.Total(aIndex: integer): ICPFlowsheetTotal;
begin
  try
    fTotals[aIndex].QueryInterface(ICPFlowsheetTotal, Result);
  except
    Result := nil;
  end;
end;

function TCPFlowsheet.TotalCount: integer;
begin
  Result := fTotals.Count;
end;

end.

