unit oCPKardexAction;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 9 $  $Modtime: 5/22/09 1:00p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary Care Action Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPKardexAction.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oCPTypes,
  oCliO,
  oCliOTypes,
  oInterfaces,
  SysUtils,
  iCliOGrid,
  XMLDoc,
  XMLIntf;

type
  TCPKardexAction = class(TInterfacedObject, ICPKardexAction, ICPGUILoader, IGridTerm, ICliOReportItem)
  protected
    fCPKardexService: ICPKardexService;
    fCPService: ICPService;
    fTerminology: ITerminology;
    fID: string;
    fCareAction: ITerm;
    fCareActionID: string;
    fSchedule: ICPSchedule;
    fScheduleType: TCPScheduleType;
    fScheduleTypePRN: TCPScheduleTypePRN;
    fPatient: IPatient;
    fStartDateTime: TDateTime;
    fStopDateTime: TDateTime;
    fFirstTime: string;
    fStatus: TCPKardexActionStatus;
    fSpecialInstructions: string;
    fScheduleTimes: string;
    fPRNTimes: integer;
    fEvents: IInterfaceList;

    function getID: string; stdcall;
    function getCareAction: ITerm; stdcall;
    function getCareActionID: string; stdcall;
    function getSchedule: ICPSchedule; stdcall;
    function getScheduleID: string; stdcall;
    function getScheduleDisplay: string; stdcall;
    function getPatient: IPatient; stdcall;
    function getStartDateTime: TDateTime; stdcall;
    function getStopDateTime: TDateTime; stdcall;
    function getFirstTime: string; stdcall;
    function getScheduleType: TCPScheduleType; stdcall;
    function getSchedulePRNType: TCPScheduleTypePRN; stdcall;
    function getStatus: TCPKardexActionStatus; stdcall;
    function getSpecialInstructions: string; stdcall;
    function getPRNNTE: integer; stdcall;
    function getScheduleTimes: string; stdcall;
    function getDuration: integer; stdcall;
    function getLastEvent: ICPKardexEvent; stdcall;
    function getActive: boolean; stdcall;

    function getEventCount: integer; stdcall;
    function getEvent(aIndex: integer): ICPKardexEvent; stdcall;
    function getFirstEvent: ICPKardexEvent; stdcall;
    function getLastCompletedEvent: ICPKardexEvent; stdcall;
    procedure AddEvent(aCPKardexEvent: ICPKardexEvent); stdcall;
    procedure ClearEvents; stdcall;

    {ICliOReportItem}
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;

    {IGridTerm}
    function getDisplayName: string; stdcall;
    function getDisplayWidth: integer; stdcall;
    function getDisplayTotal: boolean; stdcall;
    function getDisplayCount: boolean; stdcall;
    function getDisplayAverage: boolean; stdcall;
    function getDisplayMinValue: boolean; stdcall;
    function getDisplayMaxValue: boolean; stdcall;
    function getDisplayOnly: boolean; stdcall;
    function getDisplayInCellTotal: boolean; stdcall;

    procedure setTask(const aValue: ITerm); stdcall;
    procedure setSchedule(const aValue: ICPSchedule); stdcall;
    procedure setStartDateTime(const aValue: TDateTime); stdcall;
    procedure setStopDateTime(const aValue: TDateTime); stdcall;
    procedure setFirstTime(const aValue: string); stdcall;
    procedure setScheduleType(const aValue: TCPScheduleType); stdcall;
    procedure setSchedulePRNType(const aValue: TCPScheduleTypePRN); stdcall;
    procedure setStatus(const aValue: TCPKardexActionStatus); stdcall;
    procedure setSpecialInstructions(const aValue: string); stdcall;
    procedure setPRNNTE(const aValue: integer); stdcall;
    procedure setScheduleTimes(const aValue: string); stdcall;

    procedure BuildSchedule; stdcall;
    procedure ExtendSchedule(aNewStopDate: TDateTime); stdcall;
    procedure LoadSchedule(aList: TStrings); stdcall;
    procedure Save; stdcall;

    procedure DebugDisplay; stdcall;

    procedure Load;
    function LoadFromXML(aXMLNode: IXMLNode): boolean;

    {ICPGUILoader}
    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
  public
    constructor Create(aXMLNode: IXMLNode; aPatient: IPatient);
    destructor Destroy; override;
  end;

implementation

uses
  Dialogs,
  DateUtils,
  oCP_Exceptions,
  oCPKardexEvent,
  uCPXML;

procedure TCPKardexAction.ClearEvents;
begin
  fEvents.Clear;
end;

constructor TCPKardexAction.Create(aXMLNode: IXMLNode; aPatient: IPatient);
begin
  inherited Create;
  fPatient := aPatient;

  TInterfaceList.Create.GetInterface(IInterfaceList, fEvents);

  if Assigned(aXMLNode) then
    LoadFromXML(aXMLNode)
  else
    begin
      fID := NewGUID;
      fCareAction := nil;
      fSchedule := nil;
      fStartDateTime := Now;
      fStopDateTime := Now + 14;
      fStatus := asActive;
      fSpecialInstructions := '';
      fFirstTime := '0800';
    end;
end;

procedure TCPKardexAction.DebugDisplay;
var
  i: integer;
  aEvent: ICPKardexEvent;
begin
  with TStringList.Create do
    try
      Add('Start Date Time: ' + FormatDateTime('YYYY-MM-DD hh:mm', getStartDateTime));
      Add('Stop Date Time: ' + FormatDateTime('YYYY-MM-DD hh:mm', getStopDateTime));
      for i := 0 to fEvents.Count - 1 do
        if fEvents[i].QueryInterface(ICPKardexEvent, aEvent) = 0 then
          begin
            Add(
              FormatDateTime('YYYY-MM-DD hh:mm', aEvent.getScheduledDateTime) +
              '  Status: ' +
              CP_KARDEX_EVENT_STATUS[aEvent.getStatus]);
          end;
      ShowMessage('Kardex Action Display:' + #13 + Text);
    finally
      Free;
    end;
end;

destructor TCPKardexAction.Destroy;
begin
  fEvents.Clear;
  inherited;
end;

procedure TCPKardexAction.AddEvent(aCPKardexEvent: ICPKardexEvent);
begin
  fEvents.Add(aCPKardexEvent);
end;

procedure TCPKardexAction.AddToList(aList: TStrings);
begin
  aList.AddObject(fCareAction.getTerm, Self);
end;

procedure TCPKardexAction.AddToListView(aListView: TListView);
begin
  with aListView.Items.Add do
    begin
      Caption := fCareAction.getTerm;
      Data := Self;
    end;
end;

procedure TCPKardexAction.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  aText: TStringList;
  i: integer;
  aEvent: ICliOReportItem;
begin
  aText := TStringList.Create;
  try
    aReportText.Add('[' + getDisplayName + ']');
    aReportText.Add(Format('  %-15s %s to %s %d Day(s)',
      ['Duration:', FormatDateTime('YYYY-MM-DD hh:mm', fStartDateTime),
      FormatDateTime('YYYY-MM-DD hh:mm', fStopDateTime),
        DaysBetween(fStartDateTime, fStopDateTime)]));

    WrapThisValue(getScheduleDisplay, (aMargin - 25), aText);
    for i := 0 to aText.Count - 1 do
      if i = 0 then
        aReportText.Add(Format('  %-15s %s', ['Schedule:', aText[0]]))
      else
        aReportText.Add(Format('  %-15s %s', ['', aText[i]]));

    WrapThisValue(fSpecialInstructions, (aMargin - 25), aText);
    for i := 0 to aText.Count - 1 do
      if i = 0 then
        aReportText.Add(Format('  %-15s %s', ['Special Inst:', aText[0]]))
      else
        aReportText.Add(Format('  %-15s %s', ['', aText[i]]));

    aReportText.Add('');
    aReportText.Add('  ' + StringOfChar('-', aMargin - 2));
    aReportText.Add('  ' + Format('%-20s  %-18s  %-20s', ['Due Date/Time', 'Status', 'Completed']));
    aReportText.Add('  ' + StringOfChar('-', aMargin - 2));
    if getEventCount = 0 then
      aReportText.Add('    - No events documented for this time frame -')
    else
      for i := 0 to getEventCount - 1 do
        if getEvent(i).getStatus <> esPending then
          if getEvent(i).QueryInterface(ICliOReportItem, aEvent) = 0 then
            aEvent.AddToReport(aReportText, aMargin, aIncludeDetail, aList)
          else
            aReportText.Add('Unable to query interface on event.');
    aReportText.Add('');
    aReportText.Add('');
  finally
    FreeAndNil(aText);
  end;
end;

procedure TCPKardexAction.ExtendSchedule(aNewStopDate: TDateTime);
var
  i: integer;
  aEvent: ICPKardexEvent;
  aDate: TDateTime;
  aTime: TDateTime;
  h, m: Word;
  aTimes: TStringList;
  aDebug: TStringList;
begin
  if fScheduleType = stContinuous then
    try
      aDate := getStopDateTime;
      aTimes := TStringList.Create;
      aDebug := TStringList.Create;
      aTimes.CommaText := getScheduleTimes;

      while aDate <= aNewStopDate do
        begin
          for i := 0 to aTimes.Count - 1 do
            begin
              h := StrToInt(Copy(aTimes[i], 1, 2));
              m := StrToInt(Copy(aTimes[i], 3, 2));
              aTime := trunc(aDate) + EncodeTime(h, m, 0, 0);
              if (aTime >= fStopDateTime) and (aTime <= aNewStopDate) then
                with TCPKardexEvent.Create do
                  begin
                    setActionID(fID);
                    setScheduledDateTime(aTime);
                    setStatus(esPending);
                    if GetInterface(ICPKardexEvent, aEvent) then
                      fEvents.Add(aEvent);
                    aEvent.AddToList(aDebug);
                  end;
            end;
          if fSchedule.getMultiDay then
            {Check for single day or some goofball thing like 36 hours}
            aDate := incDay(aDate, trunc(fSchedule.getScheduleInterval / 1440))
          else
            aDate := IncDay(aDate, 1);
        end;
    except
      //ShowMessage('CRAP! TCPKardexAction.BuildSchedule Blew the heck up.');
    end;
  setStopDateTime(aNewStopDate);
end;

procedure TCPKardexAction.BuildSchedule;
var
  i: integer;
  aEvent: ICPKardexEvent;
  aDate: TDateTime;
  aTime: TDateTime;
  h, m: Word;
  aTimes: TStringList;
  aDebug: TStringList;
begin
  {Don't forget to find the previous entries}
  for i := fEvents.Count - 1 downto 0 do
    if (fEvents[i] as ICPKardexEvent).getStatus = esPending then
      fEvents.Delete(i);

  case fScheduleType of
    stContinuous:
      try
        aDate := trunc(fStartDateTime);
        aTimes := TStringList.Create;

        aDebug := TStringList.Create;

        aTimes.CommaText := getScheduleTimes;

        while aDate < fStopDateTime do
          begin
            for i := 0 to aTimes.Count - 1 do
              begin
                h := StrToInt(Copy(aTimes[i], 1, 2));
                m := StrToInt(Copy(aTimes[i], 3, 2));
                aTime := trunc(aDate) + EncodeTime(h, m, 0, 0);
                if (aTime >= fStartDateTime) and (aTime <= fStopDateTime) then
                  with TCPKardexEvent.Create do
                    begin
                      setActionID(fID);
                      setScheduledDateTime(aTime);
                      setStatus(esPending);
                      if GetInterface(ICPKardexEvent, aEvent) then
                        fEvents.Add(aEvent);
                      aEvent.AddToList(aDebug);
                    end;
              end;
            if fSchedule.getMultiDay then
              {Check for single day or some goofball thing like 36 hours}
              aDate := incDay(aDate, trunc(fSchedule.getScheduleInterval / 1440))
            else
              aDate := IncDay(aDate, 1);
          end;
      except
        //ShowMessage('CRAP! TCPKardexAction.BuildSchedule Blew the heck up.');
      end;
    stOneTime..stNow:
      begin
      end;
  else
    //
  end;
end;

function TCPKardexAction.getScheduleTimes: string;
begin
  if fScheduleTimes <> '' then
    Result := fScheduleTimes
  else if Assigned(fSchedule) then
    Result := fSchedule.getSchedule
  else
    Result := '';
end;

function TCPKardexAction.getDisplayAverage: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayCount: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayInCellTotal: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayMaxValue: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayMinValue: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayName: string;
begin
  Result := Format('%s (%s)', [fCareAction.getDisplayName, Copy(CP_KARDEX_ACTION_STATUS[fStatus], 1, 1)]);
end;

function TCPKardexAction.getDisplayOnly: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayTotal: boolean;
begin
  Result := False;
end;

function TCPKardexAction.getDisplayWidth: integer;
begin
  Result := 120;
end;

function TCPKardexAction.getDuration: integer;
begin
  Result := trunc(fStopDateTime - fStartDateTime);
end;

function TCPKardexAction.getEvent(aIndex: integer): ICPKardexEvent;
begin
  try
    fEvents[aIndex].QueryInterface(ICPKardexEvent, Result);
  except
    Result := nil;
  end;
end;

function TCPKardexAction.getEventCount: integer;
begin
  Result := fEvents.Count;
end;

function TCPKardexAction.getFirstEvent: ICPKardexEvent;
var
  i: integer;
begin
  if fEvents.Count > 0 then
    begin
      fEvents[0].QueryInterface(ICPKardexEvent, Result);
      for i := 1 to fEvents.Count - 1 do
        if (fEvents[i] as ICPKardexEvent).getScheduledDateTime < Result.getScheduledDateTime then
          fEvents[i].QueryInterface(ICPKardexEvent, Result);
    end
  else
    Result := nil;
end;

function TCPKardexAction.getFirstTime: string;
begin
  Result := fFirstTime;
end;

function TCPKardexAction.getID: string;
begin
  Result := fID;
end;

function TCPKardexAction.getLastCompletedEvent: ICPKardexEvent;
var
  i: integer;
begin
  if fEvents.Count > 0 then
    for i := fEvents.Count - 1 downto 0 do
      if fEvents[i].QueryInterface(ICPKardexEvent, Result) = 0 then
        if Result.getStatus = esCompleted then
          Exit;
  Result := nil;
end;

function TCPKardexAction.getLastEvent: ICPKardexEvent;
var
  i: integer;
begin
  Result := nil;
  if fEvents.Count > 0 then
    begin
      fEvents[0].QueryInterface(ICPKardexEvent, Result);
      for i := 1 to fEvents.Count - 1 do
        if (fEvents[i] as ICPKardexEvent).getScheduledDateTime > Result.getScheduledDateTime then
          fEvents[i].QueryInterface(ICPKardexEvent, Result);
    end;
end;

function TCPKardexAction.getPatient: IPatient;
begin
  Result := fPatient;
end;

function TCPKardexAction.getPRNNTE: integer;
begin
  Result := fPRNTimes;
end;

function TCPKardexAction.getSchedule: ICPSchedule;
begin
  Result := fSchedule;
end;

function TCPKardexAction.getScheduleDisplay: string;
begin
  Result := CP_SCHEDULE_TYPE[fScheduleType];
  case fScheduleType of
    stContinuous:
      begin
        if Assigned(fSchedule) then
          Result := Format('%s - %s', [Result, fSchedule.getName]);
        if fScheduleTypePRN <> prnNot then
          Result := Format('%s (%s)', [Result, CP_SCHEDULE_TYPE_PRN[fScheduleTypePRN]]);
        if fScheduleTypePRN = prnAndNTE then
          Result := Format('%s %d times', [Result, fPRNTimes]);
        if fScheduleTimes <> '' then
          Result := Result + ' (' + StringReplace(fScheduleTimes, ',', ', ', [rfReplaceAll]) + ')';
      end;
    stPRN:
      begin
        if fPRNTimes > 0 then
          Result := Format('%s - NTE %d time(s)', [fPRNTimes]);
      end;
    stOneTime:
      begin
      end;
    stStat:
      begin
      end;
    stNow:
      begin
      end;
  else
    Result := 'Unknown Schedule';
  end;
end;

function TCPKardexAction.getScheduleID: string;
begin
  if Assigned(fSchedule) then
    Result := fSchedule.getID
  else
    Result := '';
end;

function TCPKardexAction.getSchedulePRNType: TCPScheduleTypePRN;
begin
  Result := fScheduleTypePRN;
end;

function TCPKardexAction.getScheduleType: TCPScheduleType;
begin
  Result := fScheduleType;
end;

function TCPKardexAction.getSpecialInstructions: string;
begin
  Result := fSpecialInstructions;
end;

function TCPKardexAction.getStartDateTime: TDateTime;
begin
  Result := fStartDateTime;
end;

function TCPKardexAction.getStatus: TCPKardexActionStatus;
begin
  Result := fStatus;
end;

function TCPKardexAction.getStopDateTime: TDateTime;
begin
  Result := fStopDateTime;
end;

function TCPKardexAction.getActive: boolean;
begin
  result := false;
  case fScheduleType of
    stContinuous:
      begin
        Result := (fStopDateTime >= Now) and (fStatus = asActive);
      end;
    stPRN:
      begin
        if (fStopDateTime >= Now) and (fStatus = asActive) then
          if fScheduleTypePRN = prnAndNTE then
            Result := (fPRNTimes > fEvents.Count)
          else
            Result := fStopDateTime >= Now
        else
          Result := False;
      end;
    stOneTime..stNow:
      begin
        if (fStatus = asActive) then
          Result := (fEvents.Count = 0)
        else
          Result := False;
      end;
  end;
end;

function TCPKardexAction.getCareAction: ITerm;
begin
  Result := fCareAction;
end;

function TCPKardexAction.getCareActionID: string;
begin
  if Assigned(fCareAction) then
    Result := fCareAction.getID
  else
    Result := '';
end;

procedure TCPKardexAction.Load;
//var
//  aXMLDoc: IXMLDocument;
//  aXMLRec: IXMLNode;
//  aPage: ICPFlowsheetPage;
begin
  {
  try
    fIsLoaded := False;
    fPages.Clear;

    aXMLDoc := NewXMLDocument;
    if not fDBConnection.ExecQuery('GetFlowsheet', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Query ''GetFlowsheet'' failed for ID ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    if Assigned(aXMLRec) then
      LoadFromXML(aXMLRec)
    else
      raise ECPLoadFailure.CreateFmt('No record returned for flowsheet ''%s''', [fID]);

    if not fDBConnection.ExecQuery('GetFlowsheetPageList', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Query ''GetFlowsheetPageList'' failed for ID ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    while Assigned(aXMLRec) do
      try
        aPage := NewCPFlowsheetPage(aXMLRec);
        fPages.Add(aPage);
      finally
        aXMLRec := XMLDocumentNextRecord(aXMLRec);
      end;

    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Expection ''%s''', [e.Message]);
  end;
  }
end;

function TCPKardexAction.LoadFromXML(aXMLNode: IXMLNode): Boolean;
begin
  Result := True; {Assume the best here}
  with aXMLNode do
    try
      fID := XMLValueAsStr(aXMLNode, 'ID');
      fStartDateTime := XMLValueAsDateTime(aXMLNode, 'START_DATE_TIME');
      fStopDateTime := XMLValueAsDateTime(aXMLNode, 'STOP_DATE_TIME');
      fStatus := TCPKardexActionStatus(XMLValueAsInt(aXMLNode, 'STATUS'));
      fSpecialInstructions := XMLValueAsStr(aXMLNode, 'SPECIAL_INSTRUCTIONS');
      fScheduleTimes := XMLValueAsStr(aXMLNode, 'CUSTOM_SCHEDULED_TIMES');
      fScheduleType := TCPScheduleType(XMLValueAsInt(aXMLNode, 'SCHEDULE_TYPE', 0));
      fScheduleTypePRN := TCPScheduleTypePRN(XMLValueAsInt(aXMLNode, 'PRN_TYPE', 0));
    except
      Result := False;
    end;
end;

procedure TCPKardexAction.LoadSchedule(aList: TStrings);
var
  i: integer;
begin
  aList.Clear;
  for i := 0 to fEvents.Count - 1 do
    (fEvents[i] as ICPKardexEvent).AddToList(aList);
end;

procedure TCPKardexAction.Save;
begin
  //
end;

procedure TCPKardexAction.setScheduleTimes(const aValue: string);
begin
  fScheduleTimes := aValue;
end;

procedure TCPKardexAction.setFirstTime(const aValue: string);
begin
  fFirstTime := aValue;
end;

procedure TCPKardexAction.setPRNNTE(const aValue: integer);
begin
  fPRNTimes := aValue;
end;

procedure TCPKardexAction.setSchedule(const aValue: ICPSchedule);
begin
  fSchedule := aValue;
end;

procedure TCPKardexAction.setSchedulePRNType(const aValue: TCPScheduleTypePRN);
begin
  fScheduleTypePRN := aValue;
end;

procedure TCPKardexAction.setScheduleType(const aValue: TCPScheduleType);
begin
  fScheduleType := aValue;
end;

procedure TCPKardexAction.setSpecialInstructions(const aValue: string);
begin
  fSpecialInstructions := aValue;
end;

procedure TCPKardexAction.setStartDateTime(const aValue: TDateTime);
var
  h, m, s, ms: Word;
begin
  {This strips seconds and milliseconds from a value}
  DecodeTime(aValue, h, m, s, ms);
  fStartDateTime := trunc(aValue) + EncodeTime(h, m, 0, 0);
end;

procedure TCPKardexAction.setStatus(const aValue: TCPKardexActionStatus);
begin
  fStatus := aValue;
end;

procedure TCPKardexAction.setStopDateTime(const aValue: TDateTime);
var
  h, m, s, ms: Word;
begin
  {This strips seconds and milliseconds from a value}
  DecodeTime(aValue, h, m, s, ms);
  fStopDateTime := trunc(aValue) + EncodeTime(h, m, 0, 0);
end;

procedure TCPKardexAction.setTask(const aValue: ITerm);
begin
  fCareAction := aValue;
end;

end.

