unit oCPKardexEvent;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 6 $  $Modtime: 2/02/09 10:24a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary Care Action Object
*
*       Notes:
*
*
================================================================================
* $Archive: /CP Modernization/CliO/source/Common/objects/oCPKardexEvent.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oCPTypes,
  oCliO,
  oCliOTypes,
  oInterfaces,
  iCliOGrid,
  Graphics,
  SysUtils,
  XMLDoc,
  XMLIntf;

type
  TCPKardexEvent = class(TInterfacedObject, ICPKardexEvent, IGridItem, ICliOReportItem)
  private
    fEventID: string;
    fActionID: string;
    fScheduledDateTime: TDateTime;
    fStatus: TCPKardexEventStatus;
    fCompletedDateTime: TDateTime;
    fCompletedByID: string;
    fComment: string;
    fHasAudits: boolean;
  public
    constructor Create;
    destructor Destroy; override;

    {Public methods available to oCPKardexAction}
    procedure setActionID(const aValue: string);
    function LoadFromXML(aXMLNode: IXMLNode): boolean;

    {ICliOReportItem}
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;

    {IGridItem methods}
    function getDateTime: TDateTime; stdcall;
    function getDisplayValue: string; stdcall;
    function getNumericValue: Double; stdcall;
    function getDisplayFontStyle: TFontStyles; stdcall;
    function getDisplayFontColor: TColor; stdcall;
    function getDisplayFlag: TGridFlag; stdcall;
    function getAlternateText: string; stdcall;

    {ICPKardexEvent methods}
    function getEventID: string; stdcall;
    function getActionID: string; stdcall;
    function getScheduledDateTime: TDateTime; stdcall;
    function getStatus: TCPKardexEventStatus; stdcall;
    function getComment: string; stdcall;
    function getCompletedDateTime: TDateTime; stdcall;
    function getCompletedByID: string; stdcall;
    function getHasAudits: boolean; stdcall;

    procedure setScheduledDateTime(const aValue: TDateTime); stdcall;
    procedure setStatus(const aValue: TCPKardexEventStatus); stdcall;
    procedure setComment(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
  end;

implementation

uses
  oCP_Exceptions,
  uCPXML;

const
  FULL_DATE_FORMAT = 'mmmm dd, yyyy "at" hh:nn ';


{ TCPKardexEvent }

procedure TCPKardexEvent.AddToList(aList: TStrings);
begin
  if fStatus = esCompleted then
    aList.AddObject(
      FormatDateTime('YYYY-MM-DD HH:MM', fScheduledDateTime) + '  Status: ' +
      CP_KARDEX_EVENT_STATUS[fStatus] +
      FormatDateTime('YYYY-MM-DD HH:MM', fCompletedDateTime),
      Self)
  else
    aList.AddObject(
      FormatDateTime('YYYY-MM-DD HH:MM', fScheduledDateTime) + '  Status: ' +
      CP_KARDEX_EVENT_STATUS[fStatus],
      Self)
end;

procedure TCPKardexEvent.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  aLine: string;
  aText: TStringList;
  i: integer;
begin
  try
    aText := TStringList.Create;
    aLine := Format('  %-20s  %-18s',
      [FormatDateTime('YYYY-MM-DD hh:mm', fScheduledDateTime), CP_KARDEX_EVENT_STATUS[fStatus]]);
    if fStatus = esCompleted then
      aLine := Format('%s  %-20s', [aLine, FormatDateTime('YYYY-MM-DD hh:mm', fCompletedDateTime)]);
    aReportText.Add(aLine);
    if aIncludeDetail and (fComment <> '') then
      begin
        WrapThisValue(fComment, aMargin - 20, aText);
        for i := 0 to aText.Count - 1 do
          if i = 0 then
            aReportText.Add(Format('  %18s  %s', ['Comment:', aText[i]]))
          else
            aReportText.Add(Format('  %18s  %s', ['', aText[i]]));
      end;
    FreeAndNil(aText);
  except
    aReportText.Add('Error formatting Event ' + fEventID);
    FreeAndNil(aText);
  end
end;

constructor TCPKardexEvent.Create;
begin
  inherited Create;
  fEventID := NewGUID;
end;

destructor TCPKardexEvent.Destroy;
begin
  inherited;
end;

function TCPKardexEvent.getComment: string;
begin
  Result := fComment;
end;

function TCPKardexEvent.getCompletedByID: string;
begin
  Result := fCompletedByID;
end;

function TCPKardexEvent.getCompletedDateTime: TDateTime;
begin
  Result := fCompletedDateTime;
end;

function TCPKardexEvent.getDateTime: TDateTime;
begin
  Result := getScheduledDateTime;
end;

function TCPKardexEvent.getDisplayFlag: TGridFlag;
begin
  if fComment <> '' then
    Result := gfInfo
  else
    Result := gfNone;
end;

function TCPKardexEvent.getDisplayFontColor: TColor;
begin
  Result := clWindowText;
end;

function TCPKardexEvent.getDisplayFontStyle: TFontStyles;
begin
  Result := [];
end;

function TCPKardexEvent.getDisplayValue: string;
begin
  Result := CP_KARDEX_EVENT_STATUS[fStatus];
end;

function TCPKardexEvent.getEventID: string;
begin
  Result := fEventID;
end;

function TCPKardexEvent.getHasAudits: boolean;
begin
  Result := fHasAudits;
end;

function TCPKardexEvent.getNumericValue: Double;
begin
  Result := 0;
end;

function TCPKardexEvent.getScheduledDateTime: TDateTime;
begin
  Result := fScheduledDateTime;
end;

function TCPKardexEvent.getStatus: TCPKardexEventStatus;
begin
  Result := fStatus;
end;

function TCPKardexEvent.LoadFromXML(aXMLNode: IXMLNode): boolean;
begin
  try
    fEventID := XMLValueAsStr(aXMLNode, 'EVENT_ID');
    fActionID := XMLValueAsStr(aXMLNode, 'CARE_ACTION_ID');
    fScheduledDateTime := XMLValueAsDateTime(aXMLNode, 'DUE_DATE_TIME');
    fCompletedDateTime := XMLValueAsDateTime(aXMLNode, 'COMPLETED_DATE_TIME');
    fCompletedByID := XMLValueAsStr(aXMLNode, 'COMPLETED_BY_ID');
    fStatus := TCPKardexEventStatus(XMLValueAsInt(aXMLNode, 'STATUS', 0));
    fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
    fHasAudits := (XMLValueAsInt(aXMLNode, 'AUDIT_COUNT', 0) > 0);
    Result := True;
  except
    Result := False;
  end;
end;

function TCPKardexEvent.getActionID: string;
begin
  Result := fActionID;
end;

function TCPKardexEvent.getAlternateText: string;
begin
  result := ' event scheduled at ' +
    FormatDateTime(FULL_DATE_FORMAT, fScheduledDateTime) + ' ';

  case fStatus of
  esPending: result := result + ' pending ';
  esCompleted: result := result + ' completed ';
  esHeld: result := result + ' held ';
  esRefused: result := result + ' refused ';
  end;

  if fCompletedDateTime > 0 then
    result := result + FormatDateTime(' "at" ' + FULL_DATE_FORMAT + ' ', fCompletedDateTime);
  if fHasAudits then
    result := result + ' has audits.';
end;

procedure TCPKardexEvent.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TCPKardexEvent.setScheduledDateTime(const aValue: TDateTime);
var
  h, m, s, ms: Word;
begin
  DecodeTime(aValue, h, m, s, ms);
  fScheduledDateTime := trunc(aValue) + EncodeTime(h, m, 0, 0);
end;

procedure TCPKardexEvent.setStatus(const aValue: TCPKardexEventStatus);
begin
  fStatus := aValue;
end;

procedure TCPKardexEvent.setActionID(const aValue: string);
begin
  fActionID := aValue;
end;

end.

