unit oCliOReport;

interface

uses
  Classes,
  SysUtils,
  oCliO,
  oCliOTypes,
  oCPTypes,
  oDBController_Observations,
  oInterfaces;

type
  TCPReport = class(TinterfacedObject)
  private
    fLines: TStringList;
    fCliOService: TDBController_Observations;
    fTerminology: ITerminology;
    fPatient: IPatient;
    fFlowsheet: ICPFlowsheet;
    fViews: IInterfaceList;
    fMargin: integer;
    fPageLength: integer;
    fIncludeComments: boolean;
    fReportDateTimeRange: string;
    fReportDateTimeStart: TDateTime;
    fReportDateTimeStop: TDateTime;
    fReportPrintDateTime: string;

    function getPageHeader(aPageNumber: integer; aLines: TStrings): integer;

    procedure AddLine(aLine: string);
    procedure AddViewHeader(aView: IObservationView);
    procedure AddDetailHeader(aViewTerm: IObservationViewTerm);
    procedure AddDetailLine(aObs: IObservation);

    procedure setPatient(aPatient: IPatient);
    procedure setFlowsheet(aFlowsheet: ICPFlowsheet);
    procedure setDateRange(aStartDateTime, aStopDateTime: TDateTime);

    function getPage(aPageNumber: integer; aLines: TStrings): boolean; stdcall;
    function getPageCount: integer; stdcall;

    procedure Refresh(
      aStartDateTime, aStopDateTime: TDateTime;
      aPatient: IPatient;
      aFlowsheet: ICPFlowsheet;
      aObsList: ICliOList;
      aIncludeComments: boolean = False);

  public
    constructor Create(aCliOService: TDBController_Observations; aTerminology: ITerminology);
    destructor Destroy; override;
  end;

implementation

{ TCPReport }

constructor TCPReport.Create(aCliOService: TDBController_Observations; aTerminology: ITerminology);
begin
  inherited Create;
  TInterfaceList.Create.GetInterface(IInterfaceList, fViews);
  fCliOService := aCliOService;
  fTerminology := aTerminology;
  fLines := TStringList.Create;
  fMargin := 96;
  fPageLength := 64;
end;

destructor TCPReport.Destroy;
begin
  fViews.Clear;
  fViews := nil;
  FreeAndNil(fLines);
  inherited;
end;

function TCPReport.getPage(aPageNumber: integer; aLines: TStrings): boolean;
var
  i: integer;
begin
  try
    aLines.Clear;
    getPageHeader(aPageNumber, aLines);
    for i := (aPageNumber - 1) * fPageLength to (aPageNumber * fPageLength) do
      if i < fLines.Count then
        aLines.Add(fLines[i]);
    Result := True;
  except
    on E: Exception do
      begin
        aLines.Clear;
        aLines.Add('Error generating page');
        aLines.Add('Error: ' + E.Message);
        Result := False;
      end;
  end;
end;

function TCPReport.getPageCount: integer;
begin
  Result := trunc(fLines.Count / (fPageLength - 7)) + 1;
end;

function TCPReport.getPageHeader(aPageNumber: integer; aLines: TStrings): integer;
var
  aFmt: string;
begin
  with aLines do
    begin
      Clear;
      Add(StringOfChar('=', fMargin));
      Add(StringOfChar(' ', (fMargin - Length(fFlowsheet.getDisplayName)) div 2) + fFlowsheet.getDisplayName);
      Add(StringOfChar(' ', (fMargin - Length(fReportDateTimeRange)) div 2) + fReportDateTimeRange);
      if Assigned(fPatient) then
        begin
          aFmt := Format('%%s-50%%s%d', [fMargin - 50]);
          Add(Format(aFmt, [fPatient.getDisplayLine(1), '']));
          Add(Format(aFmt, [fPatient.getDisplayLine(2), 'Page: ' + IntToStr(aPageNumber)]));
          Add(Format(aFmt, [fPatient.getDisplayLine(3), 'Date: ' + fReportPrintDateTime]));
        end;
      Add(StringOfChar('=', fMargin));
    end;
  Result := aLines.Count;
end;

procedure TCPReport.setDateRange(aStartDateTime, aStopDateTime: TDateTime);
begin
  fReportDateTimeStart := aStartDateTime;
  fReportDateTimeStop := aStopDateTime;
  fReportDateTimeRange :=
    FormatDateTime('mmm d, yyyy hh:mm', aStartDateTime) +
    ' - ' +
    FormatDateTime('mmm d, yyyy hh:mm', aStopDateTime);
end;

procedure TCPReport.setFlowsheet(aFlowsheet: ICPFlowsheet);
var
  i, j: integer;
  aPage: ICPFlowsheetPage;
  aView: IObservationView;
  aViews: IInterfaceList;
begin
  fFlowsheet := aFlowsheet;
  fFlowsheet.Load;
  fViews.Clear;

  {Get any active views for this patient and store in aViews}
  TInterfaceList.Create.GetInterface(IInterfaceList, aViews);
  fCliOService.LoadActiveSupplementalViews(fPatient.getDFN, aViews);

  for j := 0 to fFlowsheet.PageCount - 1 do
    begin
      aPage := fFlowsheet.Page(j);
      case aPage.getPageType of
        ptMandatory:
          begin
            aView := fCliOService.getView(aPage.getViewID);
            aView.Load;
            fViews.Add(aView);
          end;
        ptOptional..ptSupplemental:
          begin
            for i := 0 to aViews.Count - 1 do
              if aViews[i].QueryInterface(IObservationView, aView) = 0 then
                if aView.getPageType = aPage.getPageType then
                  if aView.getID = aPage.getViewID then
                    begin
                      aView.Load;
                      fViews.Add(aView);
                    end;
          end;
      end;
    end;
end;

procedure TCPReport.setPatient(aPatient: IPatient);
begin
  fPatient := aPatient;
end;

{****************************** REPORT GENERATION *****************************}

procedure TCPReport.Refresh(
  aStartDateTime, aStopDateTime: TDateTime;
  aPatient: IPatient;
  aFlowsheet: ICPFlowsheet;
  aObsList: ICliOList;
  aIncludeComments: boolean = False);
var
  i, j: integer;
  aView: IObservationView;
  aViewTerm: IObservationViewTerm;
  aObs: IObservation;
  aDefaultUnit: ITerm;
  aTotal: double;
  aCount: integer;

begin
  try
    setPatient(aPatient);
    setDateRange(aStartDateTime, aStopDateTime);
    setFlowsheet(aFlowsheet);
    fIncludeComments := aIncludeComments;

    fLines.Clear;

    if fFlowsheet = nil then
      AddLine('No Report Selected')
    else
      begin
        for i := 0 to fViews.Count - 1 do
          begin
            fViews[i].QueryInterface(IObservationView, aView);
            AddViewHeader(aView);

            for j := 0 to aView.getViewTermCount - 1 do
              begin
                aViewTerm := aView.getViewTermByIndex(j);
                aTotal := 0;
                aCount := 0;
                AddDetailHeader(aViewTerm);
                aDefaultUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
                aObs := aObsList.FirstObservation;
                while Assigned(aObs) do
                  begin
                    if aObs.getStatus = osVerified then
                      if aViewTerm.ContainsObservation(aObs) then
                        begin
                          if not aViewTerm.getDisplayOnly then
                            AddDetailLine(aObs);
                          if aObs.getObsTerm.getDataType in [dtNumeric] then
                            begin
                              aTotal := aTotal + fTerminology.GetConvertedValue(
                                aObs.getNumericValue,
                                aObs.getObsUnit,
                                aDefaultUnit);
                              inc(aCount);
                            end;
                        end;
                    aObs := aObsList.NextObservation(aObs);
                  end;

                if aViewTerm.getDisplayTotal or aViewTerm.getDisplayCount or aViewTerm.getDisplayAverage then
                  AddLine('');

                if aViewTerm.getDisplayTotal then
                  if Assigned(aDefaultUnit) then
                    AddLine(Format('%28s: %8.2f (%s)', ['Total', aTotal, aDefaultUnit.getAbbreviation]))
                  else
                    AddLine(Format('%28s: %8.2f', ['Total', aTotal]));

                if aViewTerm.getDisplayCount then
                  AddLine(Format('%28s: %8d', ['Count', aCount]));

                if aViewTerm.getDisplayAverage then
                  if aCount > 0 then
                    if Assigned(aDefaultUnit) then
                      AddLine(Format('%28s: %8.2f (%s)', ['Average', aTotal / aCount, aDefaultUnit.getAbbreviation]))
                    else
                      AddLine(Format('%28s: %8.2f', ['Average', aTotal / aCount]))
                  else
                    AddLine(Format('%28s: %8s', ['Average', '***']));
              end;
          end;

        AddLine('');
        AddLine(StringOfChar('=', 96));
        AddLine(StringOfChar(' ', (96 - Length('FLOWSHEET TOTALS')) div 2) + 'FLOWSHEET TOTALS');
        AddLine(StringOfChar('=', 96));
        for i := 0 to fFlowsheet.TotalCount - 1 do
          with fFlowsheet.Total(i) do
            begin
              Execute(aObsList);
              AddLine(
                Format('  %-40s %10s      %8s %10s', [getDisplayName, 'Total', 'Count', 'Average']));
              if True then {Display individual terms?}
                begin
                  AddLine('  ' + StringOfChar('-', 94));
                  for j := 0 to getTermCount - 1 do
                    AddLine(
                      Format('  %-40s %10.2f %-4s %8d %10.2f %-4s',
                      [getTerm(j).getTerm,
                      getTotalByTerm(j),
                        getDefaultUnit.getAbbreviation,
                        getCountByTerm(j),
                        getAverageByTerm(j),
                        getDefaultUnit.getAbbreviation]));
                  AddLine(
                    Format('  %-40s %10s      %8s %10s', ['', '----------', '--------', '----------']));
                end;
              AddLine(
                Format('  %-40s %10.2f %-4s %8d %10.2f %-4s',
                ['',
                getTotal,
                  getDefaultUnit.getAbbreviation,
                  getCount,
                  getAverage,
                  getDefaultUnit.getAbbreviation]));
              AddLine('');
            end;
      end;
  except
    on E: Exception do
      begin
        fLines.Clear;
        fLines.Add('Error encountered generating report text');
        fLines.Add('Error: ' + e.Message);
      end;
  end;
end;

procedure TCPReport.AddLine(aLine: string);
begin
  fLines.Add(aLine);
end;

procedure TCPReport.AddViewHeader(aView: IObservationView);
begin
  AddLine('');
  AddLine(StringOfChar('=', 96));
  AddLine(StringOfChar(' ', (96 - Length(aView.getDisplayName)) div 2) + aView.getDisplayName);
  if aView.getDefaultSetID <> '' then
    begin
      AddLine('  Date Opened: ' + FormatDateTime('mmm d, yyyy', aView.getActivatedDateTime));
      AddLine('  Comment:     ' + aView.getActivatedComment);
      if aView.getDefaultMethod <> nil then
        AddLine('  Method:      ' + aView.getDefaultMethod.getTerm);

      if aView.getDefaultPosition <> nil then
        AddLine('  Position:      ' + aView.getDefaultPosition.getTerm);

      if aView.getDefaultLocation <> nil then
        AddLine('  Location:      ' + aView.getDefaultLocation.getTerm);

      if aView.getDefaultProduct <> nil then
        AddLine('  Product:       ' + aView.getDefaultProduct.getTerm);
    end;
  AddLine(StringOfChar('=', 96));
end;

procedure TCPReport.AddDetailHeader(aViewTerm: IObservationViewTerm);
begin
  AddLine('');
  if aViewTerm.getDisplayOnly then
    AddLine(Format('[%s (Totals Only)]', [aViewTerm.getDisplayName]))
  else
    begin
      AddLine(Format('[%s]', [aViewTerm.getDisplayName]));
      AddLine(
        Format('%-20s', ['Observed Date/Time']) +
        Format('%-8s  ', ['Int']) +
        Format('%-65s', ['Value']));
    end;
  AddLine(StringOfChar('-', 96));
end;

procedure TCPReport.AddDetailLine(aObs: IObservation);
var
  i: integer;
begin
  with TStringList.Create do
    try
      Text := WrapText(aObs.getDisplayValue, 65);
      AddLine(
        Format('%-20s', [FormatDateTime('yyyy-mm-dd hh:mm', aObs.getObservedDateTime)]) +
        Format('%-8s  ', [aObs.getObservedByInitials]) +
        Format('%-65s', [Strings[0]]));
      for i := 1 to Count - 1 do
        AddLine(
          Format('%-20s', ['']) +
          Format('%-8s  ', ['']) +
          Format('%-65s', [Strings[i]]));
    finally
      Free;
    end;

  if fIncludeComments then
    if Length(aObs.getComment) > 0 then
      with TStringList.Create do
        try
          Text := WrapText('Comment: ' + aObs.getComment, 90);
          for i := 0 to Count - 1 do
            AddLine('     ' + Strings[i]);
        finally
          Free;
        end;
end;

end.

