unit oCPSchedule;

interface

uses
  Classes,
  XMLDoc,
  XMLIntf,
  ComCtrls,
  oCPTypes,
  oCP_Exceptions,
  oInterfaces;

type
  TCPSchedule = class(TInterfacedObject, ICPConsoleItem, ICPSchedule)
  protected
    fID: string;
    fName: string;
    fDisplayName: string;
    fActive: boolean;
    fScheduleInterval: integer;
    fScheduleType: TCPScheduleType;
    fMultiDay: boolean;
//    fSchedule: string; {Comma list built on the fly first time needed}
    fCustomSchedule: string; {Comma list for loading into/saving from TStrings}
    fComment: string;
    fCPService: ICPService;

    {ICPConsoleItem}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;
    function getScheduleType: TCPScheduleType; stdcall;
    function getMultiDay: boolean; stdcall;
    function getSchedule: string; stdcall;
    function getScheduleInterval: integer; stdcall;
    function getCustomSchedule: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setScheduleInterval(const aValue: integer); stdcall;
    procedure setCustomSchedule(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;

    function getScheduledTimes(aStartDateTime: TDateTime = 0): string; stdcall;

    {Other private methods}
    procedure LoadFromXML(aXMLNode: IXMLNode);
    procedure Insert;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPSchedule(aXMLNode: IXMLNode; aCPService: ICPService): TCPSchedule;

implementation

uses
  Dialogs,
  DateUtils,
  SysUtils,
  uCPXML;

function NewCPSchedule(aXMLNode: IXMLNode; aCPService: ICPService): TCPSchedule;
begin
  Result := TCPSchedule.Create;
  with Result do
    try
      fCPService := aCPService;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand('AddSchedule',
          [fID, fName, fScheduleInterval, fDisplayName, fActive]);
    except
      FreeAndNil(Result);
    end;
end;

{ TCPSchedule }

constructor TCPSchedule.Create;
begin
  inherited Create;
  fID := NewGUID;
  fName := 'NEW SCHEDULE';
  fDisplayName := 'New Schedule';
  fScheduleInterval := 480; {Default to Q8H}
  fActive := False;
end;

destructor TCPSchedule.Destroy;
begin
  fCPService := nil;
  inherited;
end;

procedure TCPSchedule.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPSchedule.AddToListView(aListView: TListView);
begin
  with aListView.Items.Add do
    begin
      Caption := fName;
      Data := Self;
    end;
end;

function TCPSchedule.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TCPSchedule.getActive: boolean;
begin
  Result := fActive;
end;

function TCPSchedule.getCanDelete: boolean;
begin
  Result := false;
end;

function TCPSchedule.getCanRename: boolean;
begin
  Result := true;
end;

function TCPSchedule.getComment: string;
begin
  Result := fComment;
end;

function TCPSchedule.getCustomSchedule: string;
begin
  Result := fCustomSchedule;
end;

function TCPSchedule.getSchedule: string;
var
  aMin: integer;
  //aInt: integer;
begin
  if fScheduleInterval = 0 then
    Result := fCustomSchedule
  else if fScheduleInterval > 1440 then
    Result := '1200' {Multi-Day}
  else
    with TStringList.Create do
      try
        aMin := 0;
        while aMin < 1440 do
          begin
            Add(FormatDateTime('hhmm', IncMinute(0, aMin)));
            inc(aMin, fScheduleInterval);
          end;
        Result := CommaText;
      finally
        Free;
      end;
  (*
  if fCustomSchedule <> '' then
    Result := fCustomSchedule
  else
    begin
      if (fScheduleInterval > 0) and (fScheduleInterval < 1440) then
        aInt := fScheduleInterval
      else
        aInt := 60; {Default Multi-Day schedule to top of the hour}

      Result := '0000';
      aMin := aInt;
      while aMin < 1440 do
        begin
          Result := Result + ',' +
            FormatDateTime('hhmm', IncMinute(0, aMin));
          inc(aMin, aInt);
        end;
    end;
  *)
end;

function TCPSchedule.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TCPSchedule.getID: string;
begin
  Result := fID;
end;

function TCPSchedule.getIsLoaded: boolean;
begin
  Result := True;
end;

function TCPSchedule.getItemType: TCPItemType;
begin
  Result := ciSchedule;
end;

function TCPSchedule.getMultiDay: boolean;
begin
  Result := fScheduleInterval >= 1440 //fMultiDay;
end;

function TCPSchedule.getName: string;
begin
  Result := fName;
end;

function TCPSchedule.getScheduledTimes(aStartDateTime: TDateTime): string;
var
  aMinutes: integer;
  aTime: TDateTime;
begin
  with TStringList.Create do
    try
      Sorted := True;
      aMinutes := 0;
      while aMinutes < 1440 do
        begin
          aTime := IncMinute(aStartDateTime, aMinutes);
          Add(FormatDateTime('hhmm', aTime));
          inc(aMinutes, fScheduleInterval);
        end;
      Result := CommaText;
    finally
      Free;
    end;
end;

function TCPSchedule.getScheduleInterval: integer;
begin
  Result := fScheduleInterval;
end;

function TCPSchedule.getScheduleType: TCPScheduleType;
begin
  Result := fScheduleType;
end;

procedure TCPSchedule.Insert;
begin

end;

procedure TCPSchedule.Load;
begin

end;

procedure TCPSchedule.LoadFromXML(aXMLNode: IXMLNode);
begin
  fID := XMLValueAsStr(aXMLNode, 'ID');
  fName := XMLValueAsStr(aXMLNode, 'NAME');
  fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
  fScheduleType := TCPScheduleType(XMLValueAsInt(aXMLNode, 'SCHEDULE_TYPE', 0));
  fScheduleInterval := XMLValueAsInt(aXMLNode, 'INTERVAL', 0);
  fMultiDay := (XMLValueAsInt(aXMLNode, 'INTERVAL', 0) = 1);
  fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
  fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
  fCustomSchedule := XMLValueAsStr(aXMLNode, 'CUSTOM_SCHEDULE');
end;

procedure TCPSchedule.Save;
begin
  //
end;

procedure TCPSchedule.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TCPSchedule.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TCPSchedule.setCustomSchedule(const aValue: string);
begin
  fCustomSchedule := aValue;
end;

procedure TCPSchedule.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

procedure TCPSchedule.setName(const aValue: string);
begin
  fName := aValue;
end;

procedure TCPSchedule.setScheduleInterval(const aValue: integer);
begin
  fScheduleInterval := aValue;
end;

end.

