unit oCPShift;

interface

uses
  Classes,
  XMLDoc,
  XMLIntf,
  ComCtrls,
  oCliO,
  oCPTypes,
  oCP_Exceptions,
  oInterfaces;

type
  TCPShift = class(TInterfacedObject, ICPConsoleItem, ICPShift)
  protected
    fID: string;
    fName: string;
    fDisplayName: string;
    fActive: boolean;
    fStartTime: TDateTime;
    fStopTime: TDateTime;
    fComment: string;
    [weak] fCPService: ICPService;

    {ICPConsoleItem}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setService(const aCPService: ICPService); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;

    {ICPShift}
    function getMilitaryTime(aTime: TDateTime): string; overload;
    function getMilitaryTime(aTime: string): TDateTime; overload;
    function getDateRange: string;

    function getStartTime: TDateTime; stdcall;
    function getStopTime: TDateTime; stdcall;
    function getLength: integer; stdcall;
    function getMultiDay: boolean; stdcall;

    procedure setStartTime(const aValue: TDateTime); stdcall;
    procedure setStopTime(const aValue: TDateTime); stdcall;

    {Other private methods}
    procedure LoadFromXML(aXMLNode: IXMLNode);
    procedure Insert;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPShift(aXMLNode: IXMLNode; aCPService: ICPService): TCPShift;

implementation

uses
  DateUtils,
  SysUtils,
  uCPXML;

function NewCPShift(aXMLNode: IXMLNode; aCPService: ICPService): TCPShift;
begin
  Result := TCPShift.Create;
  with Result do
    try
      fCPService := aCPService;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand('AddShift',
          [fID, fName, getMilitaryTime(fStartTime), getMilitaryTime(fStopTime), 0, fDisplayName, fActive, fComment]);
    except
      FreeAndNil(Result);
    end;
end;

{ TCPShift }

constructor TCPShift.Create;
begin
  inherited Create;
  fID := NewGUID;
  fName := 'NEW SHIFT';
  fDisplayName := 'New Shift';
  fStartTime := EncodeTime(8, 0, 0, 0);
  fStopTime := EncodeTime(16, 30, 0, 0);
  fComment := 'New Shift Definition';
  fActive := False;
end;

destructor TCPShift.Destroy;
begin
  fCPService := nil;
  inherited;
end;

procedure TCPShift.Insert;
begin
end;

procedure TCPShift.Save;
begin
  try
    fCPService.ExecuteDBCommand('UpdateShift',
      [fID, fName, getMilitaryTime(fStartTime), getMilitaryTime(fStopTime),
      (fStartTime > fStopTime), fDisplayName, fActive, fComment]);
  except
    raise
  end;
end;

procedure TCPShift.AddToList(aList: TStrings);
begin
  aList.AddObject(getDisplayName, Self);
end;

procedure TCPShift.AddToListView(aListView: TListView);
begin
  aListView.ViewStyle := vsReport;
  if aListView.Columns.Count = 0 then
    begin
      with aListView.Columns.Add do
        begin
          Caption := 'Shift Name';
          Width := 120;
        end;
      with aListView.Columns.Add do
        begin
          Caption := 'Start/Stop Times';
          Width := 100;
        end;
      with aListView.Columns.Add do
        begin
          Caption := 'Comment';
          Width := 50;
          AutoSize := True;
        end;
      aListView.SortType := stText;
    end;

  with aListView.Items.Add do
    begin
      Caption := fName;
      SubItems.Add(getDateRange);
      SubItems.Add(fComment);
      SubItems.Add(fID);
    end;
  aListView.AlphaSort;
end;

function TCPShift.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TCPShift.getActive: boolean;
begin
  Result := fActive;
end;

function TCPShift.getCanDelete: boolean;
begin
  Result := True;
end;

function TCPShift.getCanRename: boolean;
begin
  Result := True;
end;

function TCPShift.getComment: string;
begin
  Result := fComment;
end;

function TCPShift.getDateRange: string;
begin
  Result := FormatDateTime('h:mm am/pm', fStartTime) + '-' + FormatDateTime('h:mm am/pm', fStopTime)
end;

function TCPShift.getDisplayName: string;
begin
  Result := fDisplayName;
end;

function TCPShift.getID: string;
begin
  Result := fID;
end;

function TCPShift.getIsLoaded: boolean;
begin
  Result := True;
end;

function TCPShift.getItemType: TCPItemType;
begin
  Result := ciShift;
end;

function TCPShift.getLength: integer;
begin
  Result := MinutesBetween(fstartTime, fstopTime);
end;

function TCPShift.getMilitaryTime(aTime: TDateTime): string;
begin
  if aTime = 0 then
    Result := '2400'
  else
    Result := FormatDateTime('HHMM', aTime);
end;

function TCPShift.getMilitaryTime(aTime: string): TDateTime;
begin
  if aTime = '2400' then
    Result := EncodeTime(0, 0, 0, 0)
  else
    Result := EncodeTime(StrToIntDef(Copy(aTime, 1, 2), 0), StrToIntDef(Copy(aTime, 3, 4), 0), 0, 0);
end;

function TCPShift.getMultiDay: boolean;
begin
  Result := Frac(fStopTime) < Frac(fStartTime);
end;

function TCPShift.getName: string;
begin
  Result := fName;
end;

function TCPShift.getStartTime: TDateTime;
begin
  Result := frac(fStartTime);
end;

function TCPShift.getStopTime: TDateTime;
begin
  Result := frac(fStopTime);
end;

procedure TCPShift.Load;
begin
  // Full Load from the start
end;

procedure TCPShift.LoadFromXML(aXMLNode: IXMLNode);
begin
  fID := XMLValueAsStr(aXMLNode, 'ID');
  fName := XMLValueAsStr(aXMLNode, 'NAME');
  fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
  fStartTime := getMilitaryTime(XMLValueAsStr(aXMLNode, 'START_TIME', '0800'));
  fStopTime := getMilitaryTime(XMLValueAsStr(aXMLNode, 'STOP_TIME', '1630'));
  fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
  fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
end;

procedure TCPShift.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TCPShift.setComment(const aValue: string);
begin
  if (Length(aValue) < 251) and (Pos('^', aValue) = 0) then
    fComment := aValue
  else
    raise ECPSaveFailure.Create('Comment must be 1-250 characters with no ''^''.');
end;

procedure TCPShift.setDisplayName(const aValue: string);
begin
  if (Length(aValue) < 51) and (Pos('^', aValue) = 0) then
    fDisplayName := aValue
  else
    raise ECPSaveFailure.Create('Display must be 1-51 characters with no ''^''.');
end;

procedure TCPShift.setName(const aValue: string);
begin
  begin
    if (Length(aValue) < 31) and (Pos('^', aValue) = 0) then
      fName := aValue
    else
      raise ECPSaveFailure.Create('Name must be 1-30 characters with no ''^''.');
  end;
end;

procedure TCPShift.setService(const aCPService: ICPService);
begin
  fCPService := aCPService;
end;

procedure TCPShift.setStartTime(const aValue: TDateTime);
begin
  fStartTime := aValue;
end;

procedure TCPShift.setStopTime(const aValue: TDateTime);
begin
  fStopTime := aValue;
end;

end.

