unit oCliOReport;

interface

uses
  Classes,
  SysUtils,
  Graphics,
  Types,
  oCliO,
  oCliOTypes,
  oCPTypes,
  oInterfaces;

type
  TCliOReportSectionBreak = class(TInterfacedObject, ICliOReportItem)
  private
    fHeader: string;
    fRepeatingChar: Char;
    fAlignment: TAlignment;
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;
  end;

  TCliOReport = class(TinterfacedObject, ICliOReport)
  private
    fTitle: string;
    fLines: TStringList;
    fTerminology: ITerminology;
    fPatient: IPatient;
    fCliOLoader: ICliOLoader;
    fCliOList: ICliOList;
    fMargin: integer;
    fPageLength: integer;
    fIncludeComments: boolean;
    fReportDateTimeRange: string;
    fReportDateTimeStart: TDateTime;
    fReportDateTimeStop: TDateTime;
    fReportPrintDateTime: string;

    function getPageHeader(aPageNumber: integer; aLines: TStrings): integer;

    procedure setPatient(aPatient: IPatient);
    procedure setDateRange(aStartDateTime, aStopDateTime: TDateTime);
  public
    procedure setCliOLoader(aCliOLoader: ICliOLoader);
    procedure setTerminology(aTerminology: ITerminology);

    function getPage(aPageNumber: integer; aLines: TStrings): boolean; stdcall;
    function getPageCount: integer; stdcall;
    function getSectionBreak(aTitle: string; aRepeatingCharacter: Char; aAlign: TAlignment): ICliOReportItem; stdcall;
    function getText(aLines: TStrings): integer; stdcall;

    procedure setTitle(aTitle: string); stdcall;
    procedure setMargins(aWidth, aHeight: integer); stdcall;
    procedure Refresh(
      aStartDateTime, aStopDateTime: TDateTime;
      aPatient: IPatient;
      aReportItemList: IInterfaceList;
      aIncludeComments: boolean = False); stdcall;
    procedure PrintPage(aPageNumber: Integer; aCanvas: TCanvas; aRect: TRect); stdcall;

    constructor Create;
    destructor Destroy; override;
  end;

implementation

{ TCPReport }

constructor TCliOReport.Create;
begin
  inherited Create;
  fLines := TStringList.Create;
  fMargin := 80;
  fPageLength := 64;
  fTitle := 'TITLE';
end;

destructor TCliOReport.Destroy;
begin
  FreeAndNil(fLines);
  inherited;
end;

function TCliOReport.getPage(aPageNumber: integer; aLines: TStrings): boolean;
var
  i: integer;
begin
  try
    aLines.Clear;
    getPageHeader(aPageNumber, aLines);
    for i := (aPageNumber - 1) * fPageLength to (aPageNumber * fPageLength) do
      if i < fLines.Count then
        aLines.Add(fLines[i]);
    if aPageNumber < getPageCount then
      begin
        aLines.Add('');
        aLines.Add(StringOfChar(' ', (fMargin - Length('- Continued on Next Page -')) div 2) + '- Continued on Next Page -');
      end
    else
      begin
        while aLines.Count < (fPageLength - 2) do
          aLines.Add('');
        aLines.Add(StringOfChar(' ', (fMargin - Length('- End of Report -')) div 2) + '- End of Report -');
      end;
    Result := True;
  except
    on E: Exception do
      begin
        aLines.Clear;
        aLines.Add('Error generating page');
        aLines.Add('Error: ' + E.Message);
        Result := False;
      end;
  end;
end;

function TCliOReport.getPageCount: integer;
begin
  Result := trunc(fLines.Count / fPageLength);
  if Frac(fLines.Count / fPageLength) > 0 then
    inc(Result);
end;

function TCliOReport.getPageHeader(aPageNumber: integer; aLines: TStrings): integer;
var
  aFmt: string;
begin
  with aLines do
    begin
      Clear;
      Add(StringOfChar('=', fMargin));
      Add(StringOfChar(' ', (fMargin - Length(fTitle)) div 2) + fTitle);
      Add(' ');  {Added 2011-07-21 @ 0805/drp/CQ695}
      Add(StringOfChar(' ', (fMargin - Length(fReportDateTimeRange)) div 2) + fReportDateTimeRange);
      if Assigned(fPatient) then
        begin
          aFmt := Format('%%-50s%%%ds', [fMargin - 50]);
          Add('');
          Add(Format(aFmt, [fPatient.getDisplayLine(1), '']));
          Add(Format(aFmt, [fPatient.getDisplayLine(2), 'Page: ' + IntToStr(aPageNumber)]));
          Add(Format(aFmt, [fPatient.getDisplayLine(3), 'Date: ' + fReportPrintDateTime]));
        end;
      Add(StringOfChar('=', fMargin));
    end;
  Result := aLines.Count;
end;

function TCliOReport.getSectionBreak(aTitle: string; aRepeatingCharacter: Char; aAlign: TAlignment): ICliOReportItem;
begin
  with TCliOReportSectionBreak.Create do
    try
      fHeader := aTitle;
      fRepeatingChar := aRepeatingCharacter;
      fAlignment := aAlign;
      GetInterface(ICliOReportItem, Result);
    except
      Result := nil;
    end;
end;

function TCliOReport.getText(aLines: TStrings): integer;
begin
  aLines.Clear;
  aLines.AddStrings(fLines);
  Result := aLines.Count;
end;

procedure TCliOReport.PrintPage(aPageNumber: Integer; aCanvas: TCanvas; aRect: TRect);
var
  i: integer;
  aHdr: TStringList;
  aCol, aRow, aHeight: integer;

  procedure LineOut(aLine: string);
  begin
    aCanvas.TextOut(aCol, aRow, aLine);
    inc(aRow, aHeight);
  end;

begin
  try
    aCol := aRect.Left;
    aRow := aRect.Top;
    aHeight := aCanvas.TextHeight('|');
    aHdr := TStringList.Create;
    try
      aHdr.Clear;
      getPageHeader(aPageNumber, aHdr);
      for i := 0 to aHdr.Count - 1 do
        LineOut(aHdr[i]);
    finally
      aHdr.Clear;
    end;

    for i := ((aPageNumber - 1) * fPageLength) to ((aPageNumber * fPageLength) - 1) do
      if i < fLines.Count then
        LineOut(fLines[i]);

    while aRow < (aRect.Bottom - (aCanvas.TextHeight('|') * 2)) do
      LineOut('');

    if aPageNumber < getPageCount then
      LineOut(StringOfChar(' ', (fMargin - Length('- Continued on Next Page -')) div 2) + '- Continued on Next Page -')
    else
      LineOut(StringOfChar(' ', (fMargin - Length('- End of Report -')) div 2) + '- End of Report -');
  except
    on E: Exception do
      begin
        LineOut('');
        LineOut('Error generating page');
        LineOut('Error: ' + E.Message);
      end;
  end;
end;

procedure TCliOReport.setCliOLoader(aCliOLoader: ICliOLoader);
begin
  fCliOLoader := aCliOLoader;
  fCliOList := aCliOLoader.CreateNewList;
end;

procedure TCliOReport.setDateRange(aStartDateTime, aStopDateTime: TDateTime);
begin
  fReportDateTimeStart := aStartDateTime;
  fReportDateTimeStop := aStopDateTime;
  fReportDateTimeRange :=
    FormatDateTime('mmm d, yyyy hh:mm', aStartDateTime) +
    ' - ' +
    FormatDateTime('mmm d, yyyy hh:mm', aStopDateTime);
end;

procedure TCliOReport.setMargins(aWidth, aHeight: integer);
begin
  fMargin := aWidth;
  fPageLength := (aHeight - 8); {Gotta account for the header text}
end;

procedure TCliOReport.setPatient(aPatient: IPatient);
begin
  fPatient := aPatient;
end;

procedure TCliOReport.setTerminology(aTerminology: ITerminology);
begin
  fTerminology := aTerminology;
end;

procedure TCliOReport.setTitle(aTitle: string);
begin
  fTitle := aTitle;
end;

{****************************** REPORT GENERATION *****************************}

procedure TCliOReport.Refresh(
  aStartDateTime, aStopDateTime: TDateTime;
  aPatient: IPatient;
  aReportItemList: IInterfaceList;
  aIncludeComments: boolean = False);
var
  i: integer;
  aReportItem: ICliOReportItem;
begin
  try
    setPatient(aPatient);
    setDateRange(aStartDateTime, aStopDateTime);
    fIncludeComments := aIncludeComments;

    fCliOLoader.LoadObservationsByStatus(
      fCliOList, aPatient.getDFN, aStartDateTime, aStopDateTime, [osVerified]);

    fReportPrintDateTime := FormatDateTime('mmm d, yyyy hh:mm', Now);

    fLines.Clear;

    for i := 0 to aReportItemList.Count - 1 do
      if aReportItemList[i].QueryInterface(ICliOReportItem, aReportItem) = 0 then
        aReportItem.AddToReport(fLines, fMargin, aIncludeComments, fCliOList)
      else
        fLines.Add('Unknown report object passed to ICliOReport');
  except
    on E: Exception do
      begin
        fLines.Clear;
        fLines.Add('Error encountered generating report text');
        fLines.Add('Error: ' + e.Message);
      end;
  end;
end;

{ TCliOReportSectionBreak }

procedure TCliOReportSectionBreak.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
begin
  aReportText.Add(' ');
  aReportText.Add(StringOfChar(fRepeatingChar, aMargin));
  case fAlignment of
    taCenter: aReportText.Add(StringOfChar(' ', (aMargin - Length(fHeader)) div 2) + fHeader);
    taRightJustify: aReportText.Add(Format('%' + IntToStr(aMargin) + 's', [fHeader]));
    taLeftJustify: aReportText.Add(fHeader);
  end;
  aReportText.Add(StringOfChar(fRepeatingChar, aMargin));
end;

end.

