unit oDBController_Observations;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 32 $  $Modtime: 11/16/10 8:18a $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary controller for interacting with observations in
*                     CliO applications.
*
*       Notes:
*
================================================================================
}
interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Dialogs,
  oInterfaces,
  oCliO,
  oCliOTypes,
  oDBController,
  oCP_Exceptions,
  oCPTypes,
  oObservation,
  oObservation_Battery,
  oObservation_Validator,
  XMLDoc,
  XMLIntf;

type
  TDBController_Observations = class(TDBController, ICliOService)
  private
    fErrorList: TStringList;
    fTerminology: ITerminology;
  public
    constructor Create(aTerminologyService: ITerminology);
    destructor Destroy; override;

    {Factories}
    function getNewBattery(aPatientID, aComment: string): TObservation_Battery;
    function getNewObservation(aTermID: string; aPatientID: string; aBatteryID: string): TObservation;
    function getNewValidator: TObservation_Validator;

    function getNewFiler: ICliOFiler; stdcall;
    function getNewLoader: ICliOLoader; stdcall;
    function getNewReport(aLoader: ICliOLoader): ICliOReport; stdcall;

    {Utilities}
    function getTotal(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function getAverage(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function getCount(aList: IInterfaceList; aTerm: IInterface): integer; stdcall;
    function getMaximumValue(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function getMinimumValue(aList: IInterfaceList; aTerm: IInterface): string; stdcall;

    procedure LoadLogEntries(aList: IInterfaceList; aPatientID: string; aFromDateTime, aToDateTime: TDateTime); stdcall;
  end;

implementation

uses
  oCliOLogEntry,
  oCliOReport,
  oCliOFiler,
  oCliOLoader,
  uCPXML,
  DateUtils,
  Variants;

{------------------------------------------------------------------------------}
{------------------------- TDBController_Observations -------------------------}
{------------------------------------------------------------------------------}

constructor TDBController_Observations.Create(aTerminologyService: ITerminology);
begin
  inherited Create;
  fErrorList := TStringList.Create;
  fTerminology := aTerminologyService;
end;

destructor TDBController_Observations.Destroy;
begin
  fErrorList.Clear;
  inherited;
end;

{------------------------------------------------------------------------------}
{---------------------- Factory Methods for Utility Objects -------------------}
{------------------------------------------------------------------------------}

function TDBController_Observations.getNewBattery(aPatientID, aComment: string): TObservation_Battery;
begin
  Result := NewObservationBattery(aPatientID);
  Result.Comments := aComment;
end;

function TDBController_Observations.getNewFiler: ICliOFiler;
begin
  TCliOFiler.Create(fTerminology, DBConnection).GetInterface(ICliOFiler, Result);
end;

function TDBController_Observations.getNewLoader: ICLiOLoader;
begin
  TCliOLoader.Create(fTerminology, DBConnection).GetInterface(ICliOLoader, Result);
end;

function TDBController_Observations.getNewObservation(aTermID: string; aPatientID: string; aBatteryID: string): TObservation;
begin
  Result := nil;
end;

function TDBController_Observations.getNewReport(aLoader: ICliOLoader): ICliOReport;
begin
  with TCliOReport.Create do
    try
      setCliOLoader(aLoader);
      setTerminology(fTerminology);
      GetInterface(ICliOReport, Result);
    except
      Result := nil;
    end;
end;

function TDBController_Observations.getNewValidator: TObservation_Validator;
begin
  Result := TObservation_Validator.Create;
end;

procedure TDBController_Observations.LoadLogEntries(aList: IInterfaceList; aPatientID: string; aFromDateTime, aToDateTime: TDateTime);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  aList.Clear;
  aXMLDoc := NewXMLDocument;
  DBConnection.ExecQuery('GetObservationLog', [aPatientID, aFromDateTime, aToDateTime], aXMLDoc);
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aList.Add(NewCliOLogEntry(aXMLRec));
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;

  DBConnection.ExecQuery('GetVitalsLog', [aPatientID, aFromDateTime, aToDateTime], aXMLDoc);
  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aList.Add(NewCliOLogEntry(aXMLRec));
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;

end;

function TDBController_Observations.getAverage(aList: IInterfaceList; aTerm: IInterface): string;
var
  i: integer;
  aTotal: Double;
  aCount: integer;
  aObservation: IObservation;
  aViewTerm: ICPFlowsheetViewTerm;
  aUnit: ITerm;
begin
  if aTerm.QueryInterface(ICPFlowsheetViewTerm, aViewTerm) = 0 then
    begin
      aUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
      if aUnit = nil then
        Result := '** No Unit **'
      else
        try
          aTotal := 0.0;
          aCount := 0;
          for i := 0 to aList.Count - 1 do
            if aList[i].QueryInterface(IObservation, aObservation) = 0 then
              if aObservation.getStatus = osVerified then
                begin
                  aTotal := aTotal + fTerminology.GetConvertedValue(aObservation.getNumericValue, aObservation.getObsUnit, aUnit);
                  inc(aCount);
                end;
          if aCount > 0 then
            Result := Format('%0.2f (%s)', [aTotal / aCount, aUnit.getAbbreviation])
          else
            Result := Format('%0.2f (%s)', [0.0, aUnit.getAbbreviation]);
        except
          Result := '***';
        end
    end
  else
    Result := '';
end;

function TDBController_Observations.getCount(aList: IInterfaceList; aTerm: IInterface): integer;
var
  i: integer;
  aObs: IObservation;
  aViewTerm: ICPFlowsheetViewTerm;
begin
  Result := 0;
  if aTerm.QueryInterface(ICPFlowsheetViewTerm, aViewTerm) = 0 then
    for i := 0 to aList.Count - 1 do
      if aList[i].QueryInterface(IObservation, aObs) = 0 then
        if aViewTerm.ContainsObservation(aObs) then
          if aObs.getStatus = osVerified then
            inc(Result);
end;

function TDBController_Observations.getMaximumValue(aList: IInterfaceList; aTerm: IInterface): string;
var
  aMax: Double;
  aVal: Double;
  aCount: integer;
  aObservation: IObservation;
  aViewTerm: ICPFlowsheetViewTerm;
  aUnit: ITerm;
  i: integer;
begin
  aMax := 0; //DeLint
  
  if aTerm.QueryInterface(ICPFlowsheetViewTerm, aViewTerm) = 0 then
    begin
      aUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
      if aUnit = nil then
        Result := '** No Unit **'
      else
        try
          aCount := 0;
          for i := 0 to aList.Count - 1 do
            if aList[i].QueryInterface(IObservation, aObservation) = 0 then
              if aObservation.getStatus = osVerified then
                begin
                  aVal := fTerminology.GetConvertedValue(aObservation.getNumericValue, aObservation.getObsUnit, aUnit);
                  if aCount = 0 then
                    aMax := aVal
                  else if aVal > aMax then
                    aMax := aVal;
                  inc(aCount);
                end;
          if aCount > 0 then
            Result := Format('%0.2f (%s)', [aMax, aUnit.getAbbreviation])
          else
            Result := '***';
        except
          Result := '***';
        end
    end
  else
    Result := '';
end;

function TDBController_Observations.getMinimumValue(aList: IInterfaceList; aTerm: IInterface): string;
var
  aMin: Double;
  aVal: Double;
  aCount: integer;
  aObservation: IObservation;
  aViewTerm: ICPFlowsheetViewTerm;
  aUnit: ITerm;
  i: integer;
begin
  aMin := 0;

  if aTerm.QueryInterface(ICPFlowsheetViewTerm, aViewTerm) = 0 then
    begin
      aUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
      if aUnit = nil then
        Result := '** No Unit **'
      else
        try
          aCount := 0;
          for i := 0 to aList.Count - 1 do
            if aList[i].QueryInterface(IObservation, aObservation) = 0 then
              if aObservation.getStatus = osVerified then
                begin
                  aVal := fTerminology.GetConvertedValue(aObservation.getNumericValue, aObservation.getObsUnit, aUnit);
                  if aCount = 0 then
                    aMin := aVal
                  else if aVal < aMin then
                    aMin := aVal;
                  inc(aCount);
                end;
          if aCount > 0 then
            Result := Format('%0.2f (%s)', [aMin, aUnit.getAbbreviation])
          else
            Result := '***';
        except
          Result := '***';
        end
    end
  else
    Result := '';
end;

function TDBController_Observations.getTotal(aList: IInterfaceList; aTerm: IInterface): string;
var
  i: integer;
  aTotal: Double;
  aObservation: IObservation;
  aViewTerm: ICPFlowsheetViewTerm;
  aUnit: ITerm;
begin
  if aTerm.QueryInterface(ICPFlowsheetViewTerm, aViewTerm) = 0 then
    begin
      aUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
      if aUnit = nil then
        Result := '** No Unit **'
      else if aList.Count > 0 then
        try
          aTotal := 0.0;
          for i := 0 to aList.Count - 1 do
            if aList[i].QueryInterface(IObservation, aObservation) = 0 then
              begin
                if aObservation.getStatus = osVerified then
                  aTotal := aTotal + fTerminology.GetConvertedValue(aObservation.getNumericValue, aObservation.getObsUnit, aUnit)
              end
            else
              raise Exception.Create('Invalid Item');
          Result := Format('%0.2f (%s)', [aTotal, aUnit.getAbbreviation]);
        except
          on E: Exception do
            Result := Format('Error: %s', [e.Message]);
        end
      else
        Result := Format('%0.2f (%s)', [0.0, aUnit.getAbbreviation]);
    end;
end;

end.

