unit oDBController_Terminology;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 14 $  $Modtime: 7/28/09 4:03p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Primary Terminology Controller
*
*       Notes:        THIS OBJECT SHOULD NOT BE MODIFIED WITHOUT CONSENT OF THE
*                     ENTIRE CLIO DEVELOPMENT TEAM. THIS OBJECT IS CENTRAL TO
*                     ALL APPLICATIONS.
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oDBController_Terminology.pas $
================================================================================
}
interface

uses
  Dialogs,
  DateUtils,
  SysUtils,
  Classes,
  ComCtrls,
  oInterfaces,
  oCP_Exceptions,
  oCPTypes,
  oDBController,
  oCliOTypes,
  Variants,
  XMLDoc,
  XMLIntf;

type
  TDBController_Terminology = class(TDBController, ITerminology, ITerminologyEditor)
  protected
    fCaching: boolean;
    fCachingPathName: string;

    fTerms: IInterfaceList;
    fTermMappers: IInterfaceList;
    fTermConversions: IInterfaceList;

    fLoadErrors: TStringList;

    fImportProgress: TNotifyOnProgress;

    procedure ImportTerms(aTableNode: IXMLNode);
    procedure ImportTermQualifierPairs(aTableNode: IXMLNode);
    procedure ImportTermUnitConversions(aTableNode: IXMLNode);
    procedure ImportTermUnitPairs(aTableNode: IXMLNode);
    procedure ImportTermChildPairs(aTableNode: IXMLNode);
    procedure ImportProgress(const aMsg: string; aProgress, aProgressMax: integer);

    function getFirstTerm: ITerm;
    function getNextTerm(aTerm: ITerm): ITerm;
    function getTermLoader(aID: string): ITermLoader;

    {ITerminologyEditor Interface}
    function CreateTerm(aTermType: TTermType; aDataType: TTermDataType; aID: string = ''): ITermUpdate; stdcall;
    function UpdateTerm(aTermUpdate: ITermUpdate): boolean; stdcall;

    procedure ClearClinicalDataModel(aTerm: ITerm); stdcall;

    procedure AddTermChild(aTermChild: ITermChild); stdcall;
    procedure AddTermPair(aTermPair: ITermPair); stdcall;
    procedure AddTermUnit(aTermUnit: ITermUnit); stdcall;
    procedure AddConversion(aTermConversion: ITermConversion); stdcall;

    procedure LoadTermChildren(aTerm: ITerm; aList: IInterfaceList); stdcall;
    procedure LoadTermPairs(aTerm: ITerm; aList: IInterfaceList); stdcall;
    procedure LoadTermUnits(aTerm: ITerm; aList: IInterfaceList); stdcall;
    procedure LoadUnitConversions(aUnit: ITerm; aList: IInterfaceList); stdcall;

    procedure ImportTerminology(aXMLDoc: IXMLDocument; aNotifyOnProgress: TNotifyOnProgress = nil); stdcall;

    {ITerminology Interface}
    function GetConvertedValue(aValue: Double; aFromUnit, aToUnit: ITerm): Double; stdcall;
    function GetTerm(aID: string): ITerm; stdcall;
    function GetTermCount: Integer; stdcall;
    function GetTermIndex(aList: TStrings; aID: string): Integer; stdcall;

    function GetTermMapper(aID: string): ITermMapper; overload; stdcall;
    function GetTermMapper(aIndex: integer): ITermMapper; overload; stdcall;
    function GetTermMapper(aSourceID, aSourceApplication, aSourceVersion: string): ITermMapper; overload; stdcall;
    function GetTermMapperHL7(aSendingApplication: string): ITermMapper; stdcall;
    function GetTermMapperCount: integer; stdcall;
    function GetNewTermMapper: ITermMapper; stdcall;

    function GetCached: boolean; stdcall;
    function GetCachedPathName: string; stdcall;
    function CacheAllQueries(const aPath: string): boolean; stdcall;
    procedure CacheQuery(const aQueryName, aPath: string);

    procedure SetCaching(const aSettingsString: string); stdcall;

    procedure LoadConversionObjects(aList: TStrings); stdcall;
    procedure LoadItem(aList: TStrings; aTermID: string); stdcall;
    procedure LoadList(aList: TStrings; aTermType: TTermType; aValueType: TTermValueType = vtUnspecified; aDisplayAs: TTermDisplay = tdTerm); stdcall;
    procedure LoadTermMappers(aListView: TListView); overload; stdcall;
    procedure LoadTermMappers(aList: TStrings); overload; stdcall;
    procedure LoadListView(aListView: TListView; aTermType: TTermType); overload; stdcall;
    procedure LoadTreeView(aTreeView: TTreeView); overload; stdcall;
    procedure LoadTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aTermType: TTermType); overload; stdcall;

  public
    constructor Create;
    destructor Destroy; override;
    procedure DebugMe; stdcall;
    procedure Load; stdcall;
  end;

function NewTerminologyController: TDBController_Terminology;

implementation

uses
  mFunStr,
  oTermMapper,
  oTermPair,
  oTermUnit,
  oTermChild,
  oTermConversion,
  oTerm,
  uCPXML;

function NewTerminologyController: TDBController_Terminology;
begin
  Result := TDBController_Terminology.Create;
end;

{------------------------------------------------------------------------------}
{---------------------------- TDBController_Terminology -----------------------}
{------------------------------------------------------------------------------}

constructor TDBController_Terminology.Create;
begin
  inherited Create;
  fLoadErrors := TStringList.Create;

  TInterfaceList.Create.GetInterface(IInterfaceList, fTermConversions);
  TInterfaceList.Create.GetInterface(IInterfaceList, fTermMappers);
  TInterfaceList.Create.GetInterface(IInterfaceList, fTerms);

  fCaching := False;
  fCachingPathName := '';
end;

procedure TDBController_Terminology.DebugMe;
begin
  //Just in case
end;

destructor TDBController_Terminology.Destroy;
begin
  fTermConversions.Clear;
  fTermMappers.Clear;
  fTerms.Clear;
  fTermConversions := nil;
  fTermMappers := nil;
  fTerms := nil;
  FreeAndNil(fLoadErrors);

  inherited;
end;

function TDBController_Terminology.GetCached: boolean;
begin
  Result := fCaching;
end;

function TDBController_Terminology.GetCachedPathName: string;
begin
  Result := fCachingPathName;
end;

function TDBController_Terminology.GetConvertedValue(aValue: Double; aFromUnit, aToUnit: ITerm): Double;
var
  i: Integer;
  aID: string;
  aConvertor: ITermConversion;
begin
  if aFromUnit.getID = aToUnit.getID then
    begin
      Result := aValue;
      Exit;
    end
  else
    begin
      aID := Format('%s:%s', [aFromUnit.getID, aToUnit.getID]);
      for i := 0 to fTermConversions.Count - 1 do
        if fTermConversions[i].QueryInterface(ITermConversion, aConvertor) = 0 then
          if aConvertor.getID = aID then
            begin
              Result := aConvertor.Execute(aValue);
              Exit;
            end;
    end;

  raise ETermConversionError.CreateFmt('No conversion method exists from %s to %s',[aFromUnit.getTerm, aToUnit.getTerm]);
end;

function TDBController_Terminology.getFirstTerm: ITerm;
begin
  if fTerms.Count > 0 then
    fTerms[0].QueryInterface(ITerm, Result)
  else
    Result := nil;
end;

function TDBController_Terminology.GetNewTermMapper: ITermMapper;
begin
  TTermMapper.Create(nil, DBConnection).GetInterface(ITermMapper, Result);
  fTermMappers.Add(Result);
end;

function TDBController_Terminology.getNextTerm(aTerm: ITerm): ITerm;
var
  i: Integer;
begin
  i := fTerms.IndexOf(aTerm) + 1;
  if (i > 0) and (i < fTerms.Count) then
    fTerms[i].QueryInterface(ITerm, Result)
  else
    Result := nil;
end;

function TDBController_Terminology.GetTerm(aID: string): ITerm;
var
  i: integer;
begin
  for i := 0 to fTerms.Count - 1 do
    if (fTerms[i] as ITerm).getID = aID then
      begin
        fTerms[i].QueryInterface(ITerm, Result);
        Exit;
      end;
  Result := nil;
  (*
  Result := getFirstTerm;
  while Assigned(Result) do
    begin
      if Result.getID = aID then
        Exit
      else
        Result := getNextTerm(Result);
    end;
  *)
end;

function TDBController_Terminology.GetTermCount: Integer;
begin
  Result := fTerms.Count;
end;

function TDBController_Terminology.GetTermIndex(aList: TStrings; aID: string): Integer;
begin
  for Result := 0 to aList.Count - 1 do
    if TTerm(aList.Objects[Result]).getID = aID then
      Exit;
  Result := -1;
end;

function TDBController_Terminology.getTermLoader(aID: string): ITermLoader;
var
  i: integer;
begin
  for i := 0 to fTerms.Count - 1 do
    if (fTerms[i] as ITerm).getID = aID then
      begin
        fTerms[i].QueryInterface(ITermLoader, Result);
        Exit;
      end;
  Result := nil;
end;

function TDBController_Terminology.GetTermMapper(aIndex: integer): ITermMapper;
begin
  try
    fTermMappers[aIndex].QueryInterface(ITermMapper, Result);
    Result.Load;
  except
    Result := nil;
  end;
end;

function TDBController_Terminology.GetTermMapper(aSourceID, aSourceApplication, aSourceVersion: string): ITermMapper;
var
  i: Integer;
begin
  for i := 0 to fTermMappers.Count - 1 do
    if fTermMappers[i].QueryInterface(ITermMapper, Result) = 0 then
      if Result.getSourceID = aSourceID then
        if Result.getSourceApplication = aSourceApplication then
          if Result.getSourceVersion = aSourceVersion then
            begin
              Result.Load;
              Exit;
            end;
  Result := nil;
end;

function TDBController_Terminology.GetTermMapper(aID: string): ITermMapper;
var
  i: Integer;
begin
  for i := 0 to fTermMappers.Count - 1 do
    if fTermMappers[i].QueryInterface(ITermMapper, Result) = 0 then
      if Result.getTableID = aID then
        begin
          Result.Load;
          Exit;
        end;
  Result := nil;
end;

function TDBController_Terminology.GetTermMapperCount: integer;
begin
  Result := fTermMappers.Count;
end;

function TDBController_Terminology.GetTermMapperHL7(aSendingApplication: string): ITermMapper;
var
  i: Integer;
begin
  for i := 0 to fTermMappers.Count - 1 do
    if fTermMappers[i].QueryInterface(ITermMapper, Result) = 0 then
      if Result.getHL7SendingApplication = aSendingApplication then
        begin
          Result.Load;
          Exit;
        end;
  Result := nil;
end;

procedure TDBController_Terminology.ImportTerminology(aXMLDoc: IXMLDocument; aNotifyOnProgress: TNotifyOnProgress);
var
  aTableRoot: IXMLNode;
  aTableNode: IXMLNode;
  //aRecordNode: IXMLNode;
begin
  if Assigned(aNotifyOnProgress) then
    fImportProgress := aNotifyOnProgress
  else
    fImportProgress := ImportProgress;

  aXMLDoc.Active := True;
  aTableRoot := aXMLDoc.DocumentElement.ChildNodes.FindNode('TABLES');
  aTableNode := aTableRoot.ChildNodes.First;
  while Assigned(aTableNode) do
    try
      if aTableNode.Attributes['NAME'] = 'TERM' then
        ImportTerms(aTableNode);

      if aTableNode.Attributes['NAME'] = 'TERM_CHILD_PAIR' then
        ImportTermChildPairs(aTableNode);

      if aTableNode.Attributes['NAME'] = 'TERM_UNIT_PAIR' then
        ImportTermUnitPairs(aTableNode);

      if aTableNode.Attributes['NAME'] = 'TERM_QUALIFIER_PAIR' then
        ImportTermQualifierPairs(aTableNode);

      if aTableNode.Attributes['NAME'] = 'TERM_UNIT_CONVERSION' then
        ImportTermUnitConversions(aTableNode);

    finally
      aTableNode := aTableNode.NextSibling;
    end;
end;

procedure TDBController_Terminology.ImportTermChildPairs(aTableNode: IXMLNode);
var
  aRecordNode: IXMLNode;
  aTerm: ITerm;
  aTermChild: ITermChild;
  //fTermUpdate: ITermUpdate;
  aRecTotal, aRecCount: integer;
begin
  aRecTotal := aTableNode.ChildNodes.Count;
  aRecCount := 0;

  aRecordNode := aTableNode.ChildNodes.First;

  while Assigned(aRecordNode) do
    try
      if aRecCount < aRecTotal then
        inc(aRecCount);
      fImportProgress('Importing Term Child Pairs', aRecCount, aRecTotal);
      aTerm := GetTerm(aRecordNode.ChildValues['TERM_ID']);
      NewTermChild(aTerm, aRecordNode).GetInterface(ITermChild, aTermChild);
      AddTermChild(aTermChild); {Stuff it in the db}
      (aTerm as ITermLoader).AddTermChild(aTermChild);
    finally
      aRecordNode := aRecordNode.NextSibling;
    end;
end;

procedure TDBController_Terminology.ImportTermQualifierPairs(aTableNode: IXMLNode);
var
  aRecordNode: IXMLNode;
  aTerm: ITerm;
  aTermPair: ITermPair;
  //fTermUpdate: ITermUpdate;
  aRecTotal, aRecCount: integer;
begin
  aRecTotal := aTableNode.ChildNodes.Count;
  aRecCount := 0;

  aRecordNode := aTableNode.ChildNodes.First;

  while Assigned(aRecordNode) do
    try
      if aRecCount < aRecTotal then
        inc(aRecCount);
      fImportProgress('Importing Qualifier Pairs', aRecCount, aRecTotal);
      aTerm := GetTerm(aRecordNode.ChildValues['TERM_ID']);
      aTermPair := NewTermPair(aTerm, aRecordNode);
      AddTermPair(aTermPair); {Stuff it in the db}
      (aTerm as ITermLoader).AddTermPair(aTermPair);
    finally
      aRecordNode := aRecordNode.NextSibling;
    end;
end;

procedure TDBController_Terminology.ImportTerms(aTableNode: IXMLNode);
var
  aRecordNode: IXMLNode;
  fID: string;
  fTerm: ITerm;
  fTermUpdate: ITermUpdate;
  aRecTotal, aRecCount: integer;
begin
  aRecTotal := aTableNode.ChildNodes.Count;
  aRecCount := 0;

  aRecordNode := aTableNode.ChildNodes.First;

  while Assigned(aRecordNode) do
    try
      if aRecCount < aRecTotal then
        inc(aRecCount);
      fImportProgress('Importing Terms', aRecCount, aRecTotal);
      fID := aRecordNode.ChildValues['ID'];
      fTerm := GetTerm(fID);
      if not Assigned(fTerm) then
        begin
          fTermUpdate := CreateTerm(
            TTermType(XMLValueAsInt(aRecordNode, 'TERM_TYPE')),
            TTermDataType(XMLValueAsInt(aRecordNode, 'DATA_TYPE')),
            XMLValueAsStr(aRecordNode, 'ID'));
          fTermUpdate.QueryInterface(ITerm, fTerm);
          fTerms.Add(fTerm); {New term must be added to the list or you get BOOM later with qualifiers}
        end
      else
        fTerm.QueryInterface(ITermUpdate, fTermUpdate);

      fTermUpdate.setTerm(XMLValueAsStr(aRecordNode, 'TERM'));
      fTermUpdate.setAbbreviation(XMLValueAsStr(aRecordNode, 'ABBREVIATION'));
      fTermUpdate.setActive(XMLValueAsInt(aRecordNode, 'ACTIVE', 0) = 1);
      fTermUpdate.setDataType(TTermDataType(XMLValueAsInt(aRecordNode, 'DATA_TYPE', 0)));
      fTermUpdate.setDescription(XMLValueAsStr(aRecordNode, 'DESCRIPTION'));
      fTermUpdate.setDisplayName(XMLValueAsStr(aRecordNode, 'DISPLAY_NAME'));
      fTermUpdate.setHelpText(XMLValueAsStr(aRecordNode, 'HELP_TEXT'));
      fTermUpdate.setTermType(TTermType(XMLValueAsInt(aRecordNode, 'TERM_TYPE', 0)));
      fTermUpdate.setBooleanValueTrue(XMLValueAsStr(aRecordNode, 'BOOLEAN_VALUE_TRUE'));
      fTermUpdate.setBooleanValueFalse(XMLValueAsStr(aRecordNode, 'BOOLEAN_VALUE_FALSE'));
      fTermUpdate.setMultiSelectPickList(XMLValueAsInt(aRecordNode, 'MULTI_SELECT_PICKLIST', 0) = 1);
      UpdateTerm(fTermUpdate);
      ClearClinicalDataModel(fTermUpdate);
    finally
      aRecordNode := aRecordNode.NextSibling;
    end;
end;

procedure TDBController_Terminology.ImportTermUnitConversions(aTableNode: IXMLNode);
var
  aRecordNode: IXMLNode;
  aTerm: ITerm;
  aTermConversion: ITermConversion;
  //fTermUpdate: ITermUpdate;
  aRecTotal, aRecCount: integer;
begin
  fTermConversions.Clear;
  aRecTotal := aTableNode.ChildNodes.Count;
  aRecCount := 0;

  aRecordNode := aTableNode.ChildNodes.First;

  while Assigned(aRecordNode) do
    try
      if aRecCount < aRecTotal then
        inc(aRecCount);
      fImportProgress('Importing Unit Conversions', aRecCount, aRecTotal);
      aTerm := GetTerm(aRecordNode.ChildValues['UNIT_ID_PRE']);
      NewTermConversion(aTerm, aRecordNode).GetInterface(ITermConversion, aTermConversion);
      fTermConversions.Add(aTermConversion);
      AddConversion(aTermConversion); {Stuff it in the db}
    finally
      aRecordNode := aRecordNode.NextSibling;
    end;
end;

procedure TDBController_Terminology.ImportTermUnitPairs(aTableNode: IXMLNode);
var
  aRecordNode: IXMLNode;
  aTerm: ITerm;
  aTermUnit: ITermUnit;
  //fTermUpdate: ITermUpdate;
  aRecTotal, aRecCount: integer;
begin
  aRecTotal := aTableNode.ChildNodes.Count;
  aRecCount := 0;

  aRecordNode := aTableNode.ChildNodes.First;

  while Assigned(aRecordNode) do
    try
      if aRecCount < aRecTotal then
        inc(aRecCount);
      fImportProgress('Importing Term Unit Pairs', aRecCount, aRecTotal);
      aTerm := GetTerm(aRecordNode.ChildValues['TERM_ID']);
      NewTermUnit(aTerm, aRecordNode).GetInterface(ITermUnit, aTermUnit);
      AddTermUnit(aTermUnit); {Stuff it in the db}
      (aTerm as ITermLoader).AddTermUnit(aTermUnit);
    finally
      aRecordNode := aRecordNode.NextSibling;
    end;
end;

procedure TDBController_Terminology.ImportProgress(const aMsg: string; aProgress, aProgressMax: integer);
begin
  // In case one isn't passed in
end;

function TDBController_Terminology.CreateTerm(aTermType: TTermType; aDataType: TTermDataType; aID: string = ''): ITermUpdate;
var
  aTerm: ITerm;
begin
  Result := nil;
  if NewTerm(aTermType, aDataType, aID).GetInterface(ITerm, aTerm) then
    with aTerm do
      if DBConnection.ExecCommand(
        'AddTerm', [getID, getTerm, getAbbreviation, getDisplayName, getTermType, getDataType]) then
        begin
          fTerms.Add(aTerm);
          aTerm.QueryInterface(ITermUpdate, Result);
        end;
end;

function TDBController_Terminology.UpdateTerm(aTermUpdate: ITermUpdate): boolean;
begin
  with aTermUpdate do
    Result := DBConnection.ExecCommand(
      'UpdateTerm',
      [getID,
      getTerm,
        getAbbreviation,
        getDisplayName,
        getTermType,
        getDataType,
        getActive,
        getDescription,
        getHelpText,
        getBooleanValueTrue,
        getBooleanValueFalse,
        getMultiSelectPickList]);
end;

function TDBController_Terminology.CacheAllQueries(const aPath: string): boolean;
begin
  try
    ForceDirectories(aPath);
    CacheQuery('GetTermChildPairs', aPath);
    CacheQuery('GetTermQualifierPairs', aPath);
    CacheQuery('GetTerms', aPath);
    CacheQuery('GetTermUnitConversions', aPath);
    CacheQuery('GetTermUnitPairs', aPath);
    Result := True;
  except
    Result := False;
  end;
end;

procedure TDBController_Terminology.CacheQuery(const aQueryName, aPath: string);
var
  aDoc: IXMLDOcument;
  aFile: string;
begin
  aFile := IncludeTrailingPathDelimiter(aPath) + Format('%s.xml', [aQueryName]);
  aDoc := NewXMLDocument;
  aDoc.Active := True;
  OnNotifyControllerStatus('Caching ' + aQueryName, 0, 1);
  if DBConnection.ExecQuery(aQueryName, [], aDoc) then
    begin
      aDoc.Active := True;
      aDoc.SaveToFile(aFile);
    end
  else
    raise Exception.CreateFmt('Error Caching %s', [aQueryName]);
  OnNotifyControllerStatus('', 1, 1);
end;

procedure TDBController_Terminology.ClearClinicalDataModel(aTerm: ITerm);
var
  aTermLoader: ITermLoader;
begin
  if aTerm.QueryInterface(ITermLoader, aTermLoader) = 0 then
    aTermLoader.ClearClinicalDataModel;

  with DBConnection do
    try
      ExecCommand('DeleteTermPairsByID', [aTerm.getID]);
      ExecCommand('DeleteTermUnitPairsByID', [aTerm.getID]);
      ExecCommand('DeleteTermChildrenByID', [aTerm.getID]);
      ExecCommand('DeleteTermUnitConversionsByID', [aTerm.getID]);
    except
      raise Exception.Create('Unable to clear Clinical Data Model');
    end;
end;

procedure TDBController_Terminology.AddConversion(aTermConversion: ITermConversion);
begin
  with aTermConversion do
    DBConnection.ExecCommand(
      'AddTermUnitConversion',
      [getTermPre.getID, getTermPost.getID, getOffsetPre, getOffsetPost,
      getFactor, getDecimal, getDescription]);
end;

procedure TDBController_Terminology.AddTermChild(aTermChild: ITermChild);
begin
  with aTermChild do
    DBConnection.ExecCommand(
      'AddTermChild',
      [getParentTerm.getID, getChildOrder, getChildTerm.getID,
      getValueUnit.getID, getValueType, Ord(getValueDelimiter),
        getValueStartPosition, getValueStopPosition, getDescription]);
end;

procedure TDBController_Terminology.AddTermPair(aTermPair: ITermPair);
begin
  with aTermPair do
    DBConnection.ExecCommand(
      'AddTermPair',
      [getTerm.getID, getOrder, getQualifier.getID, getValue]);
end;

procedure TDBController_Terminology.AddTermUnit(aTermUnit: ITermUnit);
begin
  with aTermUnit do
    DBConnection.ExecCommand(
      'AddTermUnit',
      [getTerm.getID, getUnit.getID, getValidMin, getValidMax, getValidDec, getReferenceMin, getReferenceMax]);
end;

procedure TDBController_Terminology.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aTerm: ITerm;
  aTermLoader: ITermLoader;
  aTermChild: ITermChild;
  aTermUnit: ITermUnit;
  aTermPair: ITermPair;
  aTermConv: ITermConversion;
  aMapper: ITermMapper;
  //aStartDateTime: TDateTime;

  function LoadDocument(aCMD: string; aParams: array of Variant; var aDocument: IXMLDocument): boolean;
  var
    aFileName: string;
  begin
    try
      aFileName := IncludeTrailingPathDelimiter(fCachingPathName) + aCMD + '.xml';
      if fCaching and FileExists(aFileName) then
        try
          aDocument.Active := True;
          aDocument.LoadFromFile(aFileName);
          result := true;
          exit;
        except
          DBConnection.ExecQuery(aCMD, aParams, aDocument);
          result := true;
          exit;
        end
      else
      begin
        DBConnection.ExecQuery(aCMD, aParams, aDocument);
        result := true;
        exit;
      end;

      //ForceDirectories('c:\terminology');
      //aDocument.Active := True;
      //aDocument.SaveToFile(Format('c:\terminology\%s.xml', [aCMD]));
    except
      on E: Exception do
        begin
          fLoadErrors.Add(e.Message);
          Result := False;
        end;
    end;
  end;

begin
  //aStartDateTime := Now;
  fLoadErrors.Clear;

  fTermConversions.Clear;
  fTerms.Clear;

  aXMLDoc := NewXMLDocument;
  try
    if LoadDocument('GetTerms', [], aXMLDoc) then
      begin
        aXMLDoc.Active := True;
        aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
        while aXMLNode <> nil do
          begin
            try
              with NewTerm(aXMLNode) do
                begin
                  if GetInterface(ITermLoader, aTermLoader) then
                    aTermLoader.SetOwnerList(fTerms);
                  if GetInterface(ITerm, aTerm) then
                    fTerms.Add(aTerm);
                end;
            except
              begin
                fLoadErrors.Add('Error loading term:');
                fLoadErrors.Add(aXMLNode.XML);
              end;
            end;
            aXMLNode := XMLDocumentNextRecord(aXMLNode);
          end;
      end;

    {Get child term pairs}
    if LoadDocument('GetTermChildPairs', [], aXMLDoc) then
      begin
        aXMLDoc.Active := True;
        aXMLnode := XMLDocumentFirstRecord(aXMLDoc);
        while aXMLNode <> nil do
          begin
            try
              aTermLoader := getTermLoader(XMLValueAsStr(aXMLNode, 'TERM_ID'));
              NewTermChild(aTerm, aXMLNode).GetInterface(ITermChild, aTermChild);
              aTermLoader.AddTermChild(aTermChild);
            except
              begin
                fLoadErrors.Add('Error loading term child pair:');
                fLoadErrors.Add(aXMLNode.XML);
              end;
            end;
            aXMLNode := XMLDocumentNextRecord(aXMLNode);
          end;
      end;

    {Get the term qualifier pairs}
    if LoadDocument('GetTermQualifierPairs', [], aXMLDoc) then
      begin
        aXMLDoc.Active := True;
        aXMLNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('RECORD');
        while aXMLNode <> nil do
          begin
            try
              aTermLoader := getTermLoader(XMLValueAsStr(aXMLNode, 'TERM_ID'));
              aTermPair := NewTermPair(aTermLoader, aXMLNode);
              aTermLoader.AddTermPair(aTermPair);
              //if getTermLoader(XMLValueAsStr(aXMLNode, 'TERM_ID')).AddTermPair(aXMLNode) = nil then
              //  raise Exception.Create('TermPair creation unsuccessful.');
            except
              on E: Exception do
                begin
                  fLoadErrors.Add('*** Error loading term qualifier pair ***');
                  fLoadErrors.Add(Format('Message: %s', [E.Message]));
                  fLoadErrors.Add('XML Data:');
                  fLoadErrors.Add(aXMLNode.XML);
                end;
            end;
            aXMLNode := aXMLNode.NextSibling;
          end;
      end;

    {Get the term unit pairs}
    if LoadDocument('GetTermUnitPairs', [], aXMLDoc) then
      begin
        aXMLDoc.Active := True;
        aXMLNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('RECORD');
        while aXMLNode <> nil do
          begin
            try
              NewTermUnit(aTerm, aXMLNode).GetInterface(ITermUnit, aTermUnit);
              getTermLoader(XMLValueAsStr(aXMLNode, 'TERM_ID')).AddTermUnit(aTermUnit);
            except
              begin
                fLoadErrors.Add('Error loading term unit pair:');
                fLoadErrors.Add(aXMLNode.XML);
              end;
            end;
            aXMLNode := aXMLNode.NextSibling;
          end;
      end;

    {Get the unit conversions}
    if LoadDocument('GetTermUnitConversions', [], aXMLDoc) then
      begin
        aXMLDoc.Active := True;
        aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
        while aXMLNode <> nil do
          begin
            try
              aTerm := GetTerm(aXMLNode.ChildValues['UNIT_ID_PRE']);
              aTermConv := NewTermConversion(aTerm, aXMLNode);
              fTermConversions.Add(aTermConv);
            except
              begin
                fLoadErrors.Add('Error loading unit conversion:');
                fLoadErrors.Add(aXMLNode.XML);
              end;
            end;
            aXMLNode := XMLDocumentNextRecord(aXMLNode);
          end;
      end;

    {Get the mapping tables **Names Only, they load dynamically when needed**}
    if not DBConnection.ExecQuery('GetMappingTableList', [], aXMLDoc) then
      fLoadErrors.Add('Error loading mapping tables')
    else
      begin
        aXMLDoc.Active := True;
        aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
        while aXMLNode <> nil do
          begin
            try
              //if NewTerminologyMapper(aXMLNode).GetInterface(ITermMapper, aMapper) then
              if TTermMapper.Create(aXMLNode, DBConnection).GetInterface(ITermMapper, aMapper) then
                fTermMappers.Add(aMapper);
            except
              begin
                fLoadErrors.Add('Error loading mapping table:');
                fLoadErrors.Add(aXMLNode.XML);
              end;
            end;
            aXMLNode := XMLDocumentNextRecord(aXMLNode);
          end;
      end;

    if fLoadErrors.Count > 0 then
      begin
        //fLoadErrors.SaveToFile('.\LoadErrors.txt');
        raise ECPLoadTermError.Create(
          'Errors encountered loading terminology' + #13 +
          'See Terminology Error Log for details.' + #13 + #13 +
          'Logfile Name: ' +
          ExpandFileName('.\LoadErrors.txt'));
      end;

  except
    on E: ECPLoadTermError do
      raise;

    on E: ECPException do
      raise ECPLoadTermFail.CreateFmt('Error Loading terminology: %s', [E.Message]);

    on E: Exception do
      raise ECPLoadTermFail.CreateFmt('Unknown Load Error: %s', [E.message]);
  end;
end;

procedure TDBController_Terminology.LoadConversionObjects(aList: TStrings);
//var
//  i: Integer;
begin
  {
  aList.Clear;
  for i := 0 to fTermConversions.Count - 1 do
    (fTermConversions[i] as ITermConversion).AddToList(aList);
  }
end;

procedure TDBController_Terminology.LoadItem(aList: TStrings; aTermID: string);
var
  aTerm: ITerm;
begin
  aTerm := GetTerm(aTermID);
  if Assigned(aTerm) then
    aTerm.AddToList(aList, tdTerm);
end;

procedure TDBController_Terminology.LoadList(aList: TStrings; aTermType: TTermType; aValueType: TTermValueType = vtUnspecified; aDisplayAs: TTermDisplay = tdTerm);
var
  i: Integer;
  aTerm: ITerm;
begin
  aList.Clear;
  for i := 0 to fTerms.Count - 1 do
    if fTerms[i].QueryInterface(ITerm, aTerm) = 0 then
      if aTerm.getTermType = aTermType then
        aTerm.AddToList(aList, aDisplayAs);
  //if (aValueType = vtUnspecified) or (aValueType = aTerm.getValueType) then
end;

procedure TDBController_Terminology.LoadListView(aListView: TListView; aTermType: TTermType);
var
  aTerm: ITerm;
  aList: TStringList;
  i: integer;
begin
  aListView.Items.Clear;
  aList := TStringList.Create;
  LoadList(aList, aTermType);

  for i := 0 to aList.Count - 1 do
    begin
      aList.Objects[i].GetInterface(ITerm, aTerm);
      with aListView.Items.Add do
        begin
          Caption := aTerm.getTerm;
          Data := aList.Objects[i];
        end;
    end;
end;

procedure TDBController_Terminology.LoadTermMappers(aList: TStrings);
var
  i: Integer;
  aTermMapper: ICPConsoleItem;
begin
  for i := 0 to fTermMappers.Count - 1 do
    if fTermMappers[i].QueryInterface(ICPConsoleItem, aTermMapper) = 0 then
      aTermMapper.AddToList(aList);
end;

procedure TDBController_Terminology.LoadTermChildren(aTerm: ITerm; aList: IInterfaceList);
begin
  aList.Clear;
end;

procedure TDBController_Terminology.LoadTermPairs(aTerm: ITerm; aList: IInterfaceList);
begin
  aList.Clear;
end;

procedure TDBController_Terminology.LoadTermUnits(aTerm: ITerm; aList: IInterfaceList);
begin
  aList.Clear;
end;

procedure TDBController_Terminology.LoadUnitConversions(aUnit: ITerm; aList: IInterfaceList);
begin
  aList.Clear;
end;

procedure TDBController_Terminology.SetCaching(const aSettingsString: string);
begin
  if FindCmdLineSwitch('nocache') then
    begin
      fCaching := false;
      fCachingPathName := '/NoCache Command Line Parameter used';
    end
  else
    begin
      fCaching := StrToIntDef(Piece(aSettingsString, ';', 1), 0) = 1;

      if fCaching then
        fCachingPathName := Piece(aSettingsString, ';', 2)
      else
        fCachingPathName := 'No Caching set';

      if fCaching and (fCachingPathName <> '') then
        if not DirectoryExists(fCachingPathName) then
          begin
            fCaching := False;
            fCachingPathName := Format('Invalid Path: %s', [fCachingPathName]);
          end;
    end;
end;

procedure TDBController_Terminology.LoadTermMappers(aListView: TListView);
var
  i: Integer;
  aTermMapper: ITermMapper;
begin
  for i := 0 to fTermMappers.Count - 1 do
    if fTermMappers[i].QueryInterface(ITermMapper, aTermMapper) = 0 then
      with aListView.Items.Add do
        begin
          Caption := aTermMapper.getTableID;
          SubItems.Add(aTermMapper.getSourceID);
          SubItems.Add(aTermMapper.getSourceApplication);
          SubItems.Add(aTermMapper.getSourceVersion);
        end;
end;

procedure TDBController_Terminology.LoadTreeView(aTreeView: TTreeView);
var
  aTermType: TTermType;
  aNodes: array[TTermType] of TTreeNode;
begin
  aTreeView.Items.Clear;
  for aTermType := Low(TTermType) to High(TTermType) do
    begin
      aNodes[aTermType] := aTreeView.Items.AddChild(nil, TERMTYPE[aTermType]);
      LoadTreeView(aTreeView, aNodes[aTermType], aTermType);
    end;
end;

procedure TDBController_Terminology.LoadTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aTermType: TTermType);
var
  i: Integer;
  aNode: TTreeNode;
  aTerm: ITermLoader;
begin
  aNode := aTreeNode.getFirstChild;
  while Assigned(aNode) do
    begin
      aTreeView.Items.Delete(aNode);
      aNode := aTreeNode.getFirstChild;
    end;
  {
  for i := 0 to fTermList.Count - 1 do
    with TTerm(fTermList.Objects[i]) do
      if getTermType = aTermType then
        aTreeView.Items.AddChildObject(aTreeNode, getTerm, fTermList.Objects[i]);
  }
  for i := 0 to fTerms.Count - 1 do
    if fTerms[i].QueryInterface(ITermLoader, aTerm) = 0 then
      if aTerm.getTermType = aTermType then
        aTreeView.Items.AddChildObject(aTreeNode, aTerm.getTerm, aTerm.GetObject);
end;

end.

