unit oMRUList;

interface

uses
  Dialogs,
  Windows,
  Messages,
  SysUtils,
  Classes,
  Forms,
  Menus,
  IniFiles;

type
  TMRUOnClick = procedure(const aFilename: string) of object;

  TMRUList = class(TObject)
  private
    fMaxFiles: integer;
    fPopupMenu: TPopupMenu;
    fMemIniFile: TMemIniFile;
    fShowFullPath: boolean;
    fFileList: TStringList;
    fMRUClickEvent: TMRUOnClick;
    fMRUNoFileEvent: TMRUOnClick;
    procedure fOnClick(Sender: TObject); {Assigned to the menu item}
    procedure fOnMRUClickEvent(const aFileName: string);
    procedure fOnMRUNoFileEvent(const aFileName: string);
    procedure LoadFilesFromIni;
    procedure SaveFilesToIni;
  public
    constructor Create(aPopupMenu: TPopupMenu; aMemIniFile: TMemIniFile = nil);
    destructor Destroy; override;

    function AddFile(const aFilename: string): boolean;
    function RemoveFile(const aFileName: string): boolean;
    procedure RefreshList;
  published
    property MaxFiles: integer read fMaxFiles write fMaxFiles;
    property ShowFullPath: boolean read fShowFullPath write fShowFullPath;
    property PopupMenu: TPopupMenu read fPopupMenu;
    property OnMenuClick: TMRUOnClick read fMRUClickEvent write fMRUClickEvent;
    property OnFileNotExist: TMRUOnClick read fMRUNoFileEvent write fMRUNoFileEvent;
  end;

implementation

constructor TMRUList.Create(aPopupMenu: TPopupMenu; aMemIniFile: TMemIniFile = nil);
begin
  inherited Create;
  fPopupMenu := aPopupMenu;
  fFileList := TStringList.create;
  if Assigned(aMemIniFile) then
    fMemIniFile := aMemIniFile
  else
    fMemIniFile := TMemIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));

  fMaxFiles := fMemIniFile.ReadInteger('MRU_Settings', 'MaxFiles', 8);
  fShowFullPath := fMemIniFile.ReadBool('MRU_Settings', 'ShowFullPath', True);

  fMRUClickEvent := fOnMRUClickEvent;
  fMRUNoFileEvent := fOnMRUNoFileEvent;
  LoadFilesFromIni;
end;

destructor TMRUList.Destroy;
begin
  SaveFilesToIni;
  FreeAndNil(fFileList);
  inherited;
end;

procedure TMRUList.fOnClick(Sender: TObject);
var
  aFileName: string;
begin
  if Sender.ClassNameIs('TMenuItem') then
    begin
      aFileName := fFileList[(Sender as TMenuItem).Tag];
      if FileExists(aFileName) then
        fMRUClickEvent(aFileName)
      else
        fMRUNoFileEvent(aFileName);
    end;
end;

procedure TMRUList.fOnMRUClickEvent(const aFileName: string);
begin
  //
end;

procedure TMRUList.fOnMRUNoFileEvent(const aFileName: string);
begin
  //
end;

function TMRUList.AddFile(const aFilename: string): boolean;
var
  i: integer;
begin
  try
    i := fFileList.IndexOf(aFileName);
    if i > -1 then {Pull it to the top if it exists}
      fFileList.Delete(i);
    fFileList.Insert(0, aFileName);
    SaveFilesToIni;
    RefreshList;
    Result := True;
  except
    Result := False;
  end;
end;

function TMRUList.RemoveFile(const aFileName: string): boolean;
var
  i: integer;
begin
  try
    i := fFileList.IndexOf(aFileName);
    if i > -1 then
      fFileList.Delete(i);
    SaveFilesToIni;
    RefreshList;
    Result := True;
  except
    Result := False;
  end;
end;

procedure TMRUList.RefreshList;
var
  i: integer;
  aItem: TMenuItem;
begin
  fPopupMenu.Items.Clear;
  for i := 0 to fFileList.Count - 1 do
    begin
      aItem := TMenuItem.Create(fPopupMenu);
      aItem.Caption := fFileList[i];
      aItem.OnClick := fOnClick;
      aItem.Tag := i; {Stores the file index}
      fPopupMenu.Items.Add(aItem);
    end;
end;

procedure TMRUList.LoadFilesFromIni;
var
  i: integer;
  aFile: string;
begin
  fFileList.Clear;
  for i := 0 to fMaxFiles do
    try
      aFile := fMemIniFile.ReadString('MRU', IntToStr(i), '');
      if aFile <> '' then
        fFileList.Add(aFile);
    except
      raise;
    end;
  RefreshList;
end;

procedure TMRUList.SaveFilesToIni;
var
  i: integer;
begin
  try
    fMemIniFile.WriteInteger('MRU_Settings', 'MaxFiles', fMaxFiles);
    fMemIniFile.WriteBool('MRU_Settings', 'ShowFullPath', fShowFullPath);

    fMemIniFile.EraseSection('MRU');
    for i := 0 to fFileList.Count - 1 do
      fMemIniFile.WriteString('MRU', IntToStr(i), fFileList[i]);
  except
    raise;
  end;
end;

(*
procedure TMostRecentFiles.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    if fRegPath = '' then
      LoadFilesFromIni else
      LoadFilesFromReg;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent,Operation);
  if (Operation = opRemove) and (AComponent = fOwnerMenuItem) then
    fOwnerMenuItem := nil;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.LoadFilesFromReg;
var
  i: cardinal;
  s: string;
begin
  fFileList.Clear;
  with TRegistry.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    if OpenKey(fRegPath, false) then
    begin
      for i := 1 to fMaxFiles do
      begin
        if ValueExists('MRU'+inttostr(i)) then
          s := readString('MRU'+inttostr(i)) else
          break;
        fFileList.Add(s);
      end;
      CloseKey;
    end;
  finally
    free;
  end;
  RefreshList;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.LoadFilesFromIni;
var
  i: cardinal;
  LoadFrom, s: string;
begin
  fFileList.Clear;
  LoadFrom := extractfilepath(fIniFilename);
  if LoadFrom <> '' then
    LoadFrom := fIniFilename
  else if fIniFilename = '' then
    LoadFrom := changefileext(paramstr(0),'.ini') //use default ini filename
  else
    LoadFrom := ExtractfilePath(Paramstr(0))+ fIniFilename; //add a path

  if fileExists(LoadFrom) then
    with TIniFile.Create(LoadFrom) do
    try
      for i := 1 to fMaxFiles do
      begin
        s := readString('MRU List',inttostr(i),'');
        if s = '' then break;
        fFileList.Add(s);
      end;
    finally
      free;
    end;
  RefreshList;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SaveFilesToReg;
var
  i: integer;
begin
  with TRegistry.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    if OpenKey(fRegPath, true) then
    begin
      for i := 1 to fMaxFiles do
        if i > fFileList.Count then
          writeString('MRU'+inttostr(i),'') else
          writeString('MRU'+inttostr(i),fFileList[i-1]);
      CloseKey;
    end;
  finally
    free;
  end;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SaveFilesToIni;
var
  i: integer;
  SaveTo: string;
begin
  SaveTo := extractfilepath(fIniFilename);
  if SaveTo <> '' then
    SaveTo := fIniFilename
  else if fIniFilename = '' then
    SaveTo := changefileext(paramstr(0),'.ini') //use default ini filename
  else
    SaveTo := ExtractfilePath(Paramstr(0))+ fIniFilename;

  with TIniFile.Create(SaveTo) do
  try
    for i := 1 to fMaxFiles do
      if i > fFileList.Count then
        writeString('MRU List',inttostr(i),'') else
        writeString('MRU List',inttostr(i),fFileList[i-1]);
  finally
    free;
  end;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SetRegPath(const aRegPath: string);
begin
  fRegPath := aRegPath;
  //if neither designing nor loading ...
  if [csDesigning, csLoading] * ComponentState = [] then
    if fRegPath = '' then
      LoadFilesFromIni else
      LoadFilesFromReg;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SetIniFile(const aIniFile: string);
begin
  fIniFilename := aIniFile;
  //if neither designing nor loading ...
  if [csDesigning, csLoading] * ComponentState = [] then
    if fRegPath = '' then
      LoadFilesFromIni;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SetOwnerMenu(aMenuItem: TMenuItem);
begin
  fOwnerMenuItem := aMenuItem;
  //if neither designing nor loading ...
  if [csDesigning, csLoading] * ComponentState = [] then
    RefreshList;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SetMaxFiles(count: cardinal);
begin
  if count = fMaxFiles then exit;
  if count < 1 then fMaxFiles := 1
  else if count > 8 then fMaxFiles := 8
  else fMaxFiles := count;
  RefreshList;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.SetShowFullPath(value: boolean);
begin
  if value = fShowFullPath then exit;
  fShowFullPath := value;
  RefreshList;
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.RefreshList;
var
  i, menuPos: integer;

  procedure AddMRUMenuItem(index: integer; const Caption: string);
  var
    NewItem: TMenuItem;
  begin
    NewItem := TMenuItem.Create(fOwnerMenuItem);
    try
      NewItem.Caption := Caption;
      NewItem.Tag := MRU_FLAG;
      fOwnerMenuItem.Insert(index,NewItem);
      if Caption <> '-' then
        NewItem.OnClick := DoClick;
    except
      NewItem.Free;
      raise;
    end;
  end;

begin
  if (csDesigning in ComponentState) or not assigned(fOwnerMenuItem)  then exit;

  //remove all existing MRU items ...
  for i := fOwnerMenuItem.Count-1 downto 0 do
    if fOwnerMenuItem.Items[i].Tag = MRU_FLAG then fOwnerMenuItem.Delete(i);

  if (fFileList.Count = 0) then exit; //that's it

  if (integer(fMenuPosition) >= fOwnerMenuItem.Count) then
    menuPos := fOwnerMenuItem.Count else
    menuPos := fMenuPosition;

  //? add a preceeding separator item...
  if (menuPos > 0) and (fOwnerMenuItem.Items[menuPos-1].Caption <> '-') then
  begin
    AddMRUMenuItem(MenuPos,'-');
    inc(MenuPos);
  end;

  for i := 0 to fFileList.Count-1 do
  begin
    if fShowFullPath then
      AddMRUMenuItem(MenuPos, format('&%d  %s',[i+1,fFileList[i]])) else
      AddMRUMenuItem(MenuPos, format('&%d  %s',[i+1,extractFilename(fFileList[i])]));
    inc(MenuPos);
  end;

  //? add a trailing separator item...
  if (menuPos < fOwnerMenuItem.Count ) and
    (fOwnerMenuItem.Items[menuPos].Caption <> '-') then
      AddMRUMenuItem(MenuPos,'-');
end;
//------------------------------------------------------------------------------

procedure TMostRecentFiles.DoClick(Sender: TObject);
var
  i,idx: integer;
  ParentMenuItem: TMenuItem;
  s, filename: string;
begin
  if not (Sender is TMenuItem) or not assigned(fMRUClickEvent) then exit;
  ParentMenuItem := TMenuItem(Sender).Parent;
  if not assigned(ParentMenuItem) then exit;
  //get index of Sender menuItem within Parent's menuItem list
  idx := ParentMenuItem.IndexOf(TMenuItem(Sender));
  //find index of first item ...
  i := 0;
  while (i < ParentMenuItem.Count) and
    (ParentMenuItem.items[i].Tag <> MRU_FLAG) do inc(i);
  if (i = ParentMenuItem.Count) then exit; //should never happen
  if ParentMenuItem.items[i].Caption = '-' then inc(i); //skips the separator
  idx := idx -i; //this is the real index into fFileList
  if (idx < 0) or (idx >= fFileList.Count) then exit; //should never happen
  filename := fFileList[idx];
  if not fileExists(filename) then
  begin
    //no file so warn then delete it from the list ...
    if assigned(fNoFileEvent) then
      fNoFileEvent(sender, filename) //eg: may prefer just a simple beep
    else
    begin
      s := format(s_no_file_exists,[filename]);
      if assigned(Owner) and (Owner is TCustomForm) then
        MessageBox(TCustomForm(Owner).handle,
          pchar(s),pchar(application.title), mb_iconInformation)
      else
        MessageBox(0, pchar(s),pchar(application.title), mb_iconInformation);
    end;
    fFileList.Delete(idx);
    RefreshList;
  end else
  begin
    //update order before doing something ...
    if idx > 0 then
    begin
      fFileList.Delete(idx);
      fFileList.Insert(0,Filename);
      RefreshList;
    end;
    if assigned(fMRUClickEvent) then
      fMRUClickEvent(Sender, filename);
  end;
end;
//------------------------------------------------------------------------------

function TMostRecentFiles.AddFile(const Filename: string): boolean;
var
  i: integer;
begin
  result := false;
  if not assigned(fOwnerMenuItem) then exit;
  i := fFileList.IndexOf(Filename);
  if i = 0 then exit //already the first file in list
  else if i > 0 then fFileList.Delete(i);
  fFileList.Insert(0,Filename);
  while fFileList.count > integer(fMaxFiles) do fFileList.delete(fMaxFiles);
  RefreshList;
end;
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
*)
end.

